@@ -1,4 +1,4 @@
-/* $Id: color.c,v 1.5 2004/05/27 18:39:15 dolorous Exp $ */
+/* $Id: color.c,v 1.7 2004/07/31 14:10:23 dolorous Exp $ */
 /**************************************************************************
  *   color.c                                                              *
  *                                                                        *
@@ -25,7 +25,6 @@
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include "proto.h"
@@ -121,7 +120,7 @@ void update_color(void)
     if (colorstrings == NULL && syntaxstr != NULL) {
 	for (tmpsyntax = syntaxes; tmpsyntax != NULL;
 	     tmpsyntax = tmpsyntax->next) {
-	    if (!strcasecmp(tmpsyntax->desc, syntaxstr))
+	    if (strcasecmp(tmpsyntax->desc, syntaxstr) == 0)
 		colorstrings = tmpsyntax->color;
 	}
     }
