@@ -1,4 +1,4 @@
-/* $Id: files.c,v 1.37 2004/06/01 20:37:59 dolorous Exp $ */
+/* $Id: files.c,v 1.62 2004/08/17 05:23:38 dolorous Exp $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -25,7 +25,6 @@
 #include <string.h>
 #include <stdio.h>
 #include <unistd.h>
-#include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
 #include <utime.h>
@@ -86,7 +85,7 @@ void new_file(void)
        on the command line, a new file will be created, but it will be
        given no open_files entry */
     if (open_files == NULL) {
-	add_open_file(0);
+	add_open_file(FALSE);
 	/* turn off view mode in this case; this is for consistency
 	   whether multibuffers are compiled in or not */
 	UNSET(VIEW_MODE);
@@ -154,7 +153,7 @@ filestruct *read_line(char *buf, filestruct *prev, int *line1ins, size_t
     return fileptr;
 }
 
-int read_file(FILE *f, const char *filename, int quiet)
+void read_file(FILE *f, const char *filename, int quiet)
 {
     int num_lines = 0, len = 0;
     char input = '\0';		/* current input character */
@@ -336,11 +335,12 @@ int read_file(FILE *f, const char *filename, int quiet)
 
     totlines += num_lines;
 
-    return 1;
+    return;
 }
 
-/* Open the file (and decide if it exists). */
-int open_file(const char *filename, int insert, int quiet)
+/* Open the file (and decide if it exists).  Return TRUE on success,
+ * FALSE on failure. */
+bool open_file(const char *filename, int insert, int quiet)
 {
     int fd;
     FILE *f;
@@ -349,7 +349,7 @@ int open_file(const char *filename, int insert, int quiet)
     if (filename[0] == '\0' || stat(filename, &fileinfo) == -1) {
 	if (insert && !quiet) {
 	    statusbar(_("\"%s\" not found"), filename);
-	    return -1;
+	    return FALSE;
 	} else {
 	    /* We have a new file */
 	    statusbar(_("New File"));
@@ -362,7 +362,7 @@ int open_file(const char *filename, int insert, int quiet)
 			_("File \"%s\" is a device file"), filename);
 	if (!insert)
 	    new_file();
-	return -1;
+	return FALSE;
     } else if ((fd = open(filename, O_RDONLY)) == -1) {
 	/* If we're in multibuffer mode, don't be quiet when an error
 	   occurs while opening a file */
@@ -371,10 +371,10 @@ int open_file(const char *filename, int insert, int quiet)
 		|| ISSET(MULTIBUFFER)
 #endif
 		)
-	    statusbar("%s: %s", strerror(errno), filename);
+	    statusbar("Error reading %s: %s", filename, strerror(errno));
 	if (!insert)
 	    new_file();
-	return -1;
+	return FALSE;
     } else {			/* File is A-OK */
 	if (!quiet)
 	    statusbar(_("Reading File"));
@@ -382,7 +382,7 @@ int open_file(const char *filename, int insert, int quiet)
 	if (f == NULL) {
 	    nperror("fdopen");
 	    close(fd);
-	    return -1;
+	    return FALSE;
 	}
 	read_file(f, filename, quiet);
 #ifndef NANO_SMALL
@@ -390,23 +390,26 @@ int open_file(const char *filename, int insert, int quiet)
 #endif
     }
 
-    return 1;
+    return TRUE;
 }
 
 /* This function will return the name of the first available extension
- * of a filename (starting with the filename, then filename.1, etc).
- * Memory is allocated for the return value.  If no writable extension
- * exists, we return "". */
+ * of a filename (starting with the filename.save, then filename.save.1,
+ * etc).  Memory is allocated for the return value.  If no writable
+ * extension exists, we return "". */
 char *get_next_filename(const char *name)
 {
     int i = 0;
-    char *buf = NULL;
-    struct stat fs;
+    char *buf;
+    size_t namelen = strlen(name);
 
-    buf = charalloc(strlen(name) + num_of_digits(INT_MAX) + 2);
+    buf = charalloc(namelen + num_of_digits(INT_MAX) + 7);
     strcpy(buf, name);
+    strcpy(buf + namelen, ".save");
+    namelen += 5;
 
     while (TRUE) {
+	struct stat fs;
 
 	if (stat(buf, &fs) == -1)
 	    return buf;
@@ -414,8 +417,7 @@ char *get_next_filename(const char *name)
 	    break;
 
 	i++;
-	strcpy(buf, name);
-	sprintf(&buf[strlen(name)], ".%d", i);
+	sprintf(buf + namelen, ".%d", i);
     }
 
     /* We get here only if there is no possible save file. */
@@ -424,9 +426,11 @@ char *get_next_filename(const char *name)
     return buf;
 }
 
-int do_insertfile(int loading_file)
+void do_insertfile(int loading_file)
 {
     int i, old_current_x = current_x;
+    bool opened;
+	/* TRUE if the file opened successfully. */
     char *realname = NULL;
     static char *inspath = NULL;
 
@@ -439,16 +443,13 @@ int do_insertfile(int loading_file)
     wrap_reset();
 #endif
 
-#if !defined(DISABLE_BROWSER) || !defined(NANO_SMALL) && defined(ENABLE_MULTIBUFFER)
-  start_again:	/* Goto here when the user cancels the file browser. */
-#endif
-
+  start_again:
 #if !defined(DISABLE_BROWSER) || !defined(DISABLE_MOUSE)
     currshortcut = insertfile_list;
 #endif
 
 #ifndef DISABLE_OPERATINGDIR
-    if (operating_dir != NULL && strcmp(operating_dir, "."))
+    if (operating_dir != NULL && strcmp(operating_dir, ".") != 0)
 #ifdef ENABLE_MULTIBUFFER 
 	if (ISSET(MULTIBUFFER))
 	    i = statusq(TRUE, insertfile_list, inspath,
@@ -510,7 +511,7 @@ int do_insertfile(int loading_file)
 	    if (ts  == -1 || answer == NULL || answer[0] == '\0') {
 		statusbar(_("Cancelled"));
 		display_main_list();
-		return 0;
+		return;
 	    }
 	}
 #endif /* !NANO_SMALL */
@@ -532,17 +533,17 @@ int do_insertfile(int loading_file)
 #ifndef NANO_SMALL
 		i != NANO_EXTCMD_KEY &&
 #endif
-		check_operating_dir(answer, 0) != 0) {
+		check_operating_dir(answer, FALSE) != 0) {
 	    statusbar(_("Can't insert file from outside of %s"),
-			operating_dir);
-	    return 0;
+		operating_dir);
+	    return;
 	}
 #endif
 
 #ifdef ENABLE_MULTIBUFFER
 	if (loading_file) {
 	    /* update the current entry in the open_files structure */
-	    add_open_file(1);
+	    add_open_file(TRUE);
 	    new_file();
 	    UNSET(MODIFIED);
 #ifndef NANO_SMALL
@@ -554,13 +555,14 @@ int do_insertfile(int loading_file)
 #ifndef NANO_SMALL
 	if (i == NANO_EXTCMD_KEY) {
 	    realname = mallocstrcpy(realname, "");
-	    i = open_pipe(answer);
-	} else
-#endif /* NANO_SMALL */
-	{
+	    opened = open_pipe(answer);
+	} else {
+#endif
 	    realname = real_dir_from_tilde(answer);
-	    i = open_file(realname, 1, loading_file);
+	    opened = open_file(realname, TRUE, loading_file);
+#ifndef NANO_SMALL
 	}
+#endif
 
 #ifdef ENABLE_MULTIBUFFER
 	if (loading_file) {
@@ -569,7 +571,7 @@ int do_insertfile(int loading_file)
 	       created to hold the file), reload the buffer we had open
 	       before, and skip the insertion; otherwise, save realname
 	       in filename and continue the insertion */
-	    if (i == -1) {
+	    if (!opened) {
 		free(realname);
 		free(fileage);
 		load_open_file();
@@ -587,7 +589,7 @@ int do_insertfile(int loading_file)
 
 #ifdef ENABLE_MULTIBUFFER
 	if (loading_file)
-	    load_file(0);
+	    load_file(FALSE);
 	else
 #endif
 	    set_modified();
@@ -599,21 +601,12 @@ int do_insertfile(int loading_file)
 	    titlebar(NULL);
 
 	    /* And re-init the shortcut list */
-	    shortcut_init(0);
-	}
+	    shortcut_init(FALSE);
+	} else
 #endif
-
-#ifdef ENABLE_MULTIBUFFER
-	if (!loading_file) {
-#endif
-
 	    /* Restore the old x-coordinate position */
 	    current_x = old_current_x;
 
-#ifdef ENABLE_MULTIBUFFER
-	}
-#endif
-
 	/* If we've gone off the bottom, recenter; otherwise, just redraw */
 	edit_refresh();
 
@@ -630,27 +623,24 @@ int do_insertfile(int loading_file)
     inspath = NULL;
 
     display_main_list();
-    return i;
 }
 
-int do_insertfile_void(void)
+void do_insertfile_void(void)
 {
-    int result = 0;
 #ifdef ENABLE_MULTIBUFFER
     if (ISSET(VIEW_MODE)) {
 	if (ISSET(MULTIBUFFER))
-	    result = do_insertfile(1);
+	    do_insertfile(TRUE);
 	else
 	    statusbar(_("Key illegal in non-multibuffer mode"));
     }
     else
-	result = do_insertfile(ISSET(MULTIBUFFER));
+	do_insertfile(ISSET(MULTIBUFFER));
 #else
-    result = do_insertfile(0);
+    do_insertfile(FALSE);
 #endif
 
     display_main_list();
-    return result;
 }
 
 #ifdef ENABLE_MULTIBUFFER
@@ -671,7 +661,7 @@ openfilestruct *make_new_opennode(openfilestruct *prevnode)
 
 /* Splice a node into an existing openfilestruct. */
 void splice_opennode(openfilestruct *begin, openfilestruct *newnode,
-		     openfilestruct *end)
+	openfilestruct *end)
 {
     newnode->next = end;
     newnode->prev = begin;
@@ -725,15 +715,15 @@ void free_openfilestruct(openfilestruct *src)
 
 /*
  * Add/update an entry to the open_files openfilestruct.  If update is
- * zero, a new entry is created; otherwise, the current entry is updated.
- * Return 0 on success or 1 on error.
+ * FALSE, a new entry is created; otherwise, the current entry is
+ * updated.
  */
-int add_open_file(int update)
+void add_open_file(int update)
 {
     openfilestruct *tmp;
 
     if (fileage == NULL || current == NULL || filename == NULL)
-	return 1;
+	return;
 
     /* if no entries, make the first one */
     if (open_files == NULL)
@@ -808,19 +798,16 @@ int add_open_file(int update)
 #ifdef DEBUG
     fprintf(stderr, "filename is %s\n", open_files->filename);
 #endif
-
-    return 0;
 }
 
 /*
  * Read the current entry in the open_files structure and set up the
- * currently open file using that entry's information.  Return 0 on
- * success or 1 on error.
+ * currently open file using that entry's information.
  */
-int load_open_file(void)
+void load_open_file(void)
 {
     if (open_files == NULL)
-	return 1;
+	return;
 
     /* set up the filename, the file buffer, the total number of lines in
        the file, and the total file size */
@@ -856,38 +843,36 @@ int load_open_file(void)
 
     /* restore full file position: line number, x-coordinate, y-
        coordinate, place we want */
-    do_gotopos(open_files->file_lineno, open_files->file_current_x, open_files->file_current_y, open_files->file_placewewant);
+    do_gotopos(open_files->file_lineno, open_files->file_current_x,
+	open_files->file_current_y, open_files->file_placewewant);
 
     /* update the titlebar */
     clearok(topwin, FALSE);
     titlebar(NULL);
-
-    /* now we're done */
-    return 0;
 }
 
 /*
  * Open the previous entry in the open_files structure.  If closing_file
- * is zero, update the current entry before switching from it.
- * Otherwise, we are about to close that entry, so don't bother doing so.
- * Return 0 on success and 1 on error.
+ * is FALSE, update the current entry before switching from it.
+ * Otherwise, we are about to close that entry, so don't bother doing
+ * so.
  */
-int open_prevfile(int closing_file)
+void open_prevfile(int closing_file)
 {
     if (open_files == NULL)
-	return 1;
+	return;
 
     /* if we're not about to close the current entry, update it before
        doing anything */
     if (!closing_file)
-	add_open_file(1);
+	add_open_file(TRUE);
 
     if (open_files->prev == NULL && open_files->next == NULL) {
 
 	/* only one file open */
 	if (!closing_file)
 	    statusbar(_("No more open file buffers"));
-	return 1;
+	return;
     }
 
     if (open_files->prev != NULL) {
@@ -914,43 +899,40 @@ int open_prevfile(int closing_file)
     load_open_file();
 
     statusbar(_("Switched to %s"),
-      ((open_files->filename[0] == '\0') ? "New Buffer" : open_files->filename));
+      ((open_files->filename[0] == '\0') ? "New Buffer" :
+	open_files->filename));
 
 #ifdef DEBUG
     dump_buffer(current);
 #endif
-
-    return 0;
 }
 
-/* This function is used by the shortcut list. */
-int open_prevfile_void(void)
+void open_prevfile_void(void)
 {
-    return open_prevfile(0);
+    open_prevfile(FALSE);
 }
 
 /*
  * Open the next entry in the open_files structure.  If closing_file is
- * zero, update the current entry before switching from it.  Otherwise, we
- * are about to close that entry, so don't bother doing so.  Return 0 on
- * success and 1 on error.
+ * FALSE, update the current entry before switching from it.  Otherwise,
+ * we are about to close that entry, so don't bother doing so.
  */
-int open_nextfile(int closing_file)
+void open_nextfile(int closing_file)
 {
     if (open_files == NULL)
-	return 1;
+	return;
 
     /* if we're not about to close the current entry, update it before
        doing anything */
     if (!closing_file)
-	add_open_file(1);
+	add_open_file(TRUE);
 
     if (open_files->prev == NULL && open_files->next == NULL) {
 
 	/* only one file open */
 	if (!closing_file)
 	    statusbar(_("No more open file buffers"));
-	return 1;
+	return;
     }
 
     if (open_files->next != NULL) {
@@ -977,19 +959,17 @@ int open_nextfile(int closing_file)
     load_open_file();
 
     statusbar(_("Switched to %s"),
-      ((open_files->filename[0] == '\0') ? "New Buffer" : open_files->filename));
+      ((open_files->filename[0] == '\0') ? "New Buffer" :
+	open_files->filename));
 
 #ifdef DEBUG
     dump_buffer(current);
 #endif
-
-    return 0;
 }
 
-/* This function is used by the shortcut list. */
-int open_nextfile_void(void)
+void open_nextfile_void(void)
 {
-    return open_nextfile(0);
+    open_nextfile(FALSE);
 }
 
 /*
@@ -1011,15 +991,17 @@ int close_open_file(void)
     open_files->filebot = filebot;
 
     tmp = open_files;
-    if (open_nextfile(1)) {
-	if (open_prevfile(1))
-	    return 1;
-    }
+    if (open_files->next != NULL)
+	open_nextfile(TRUE);
+    else if (open_files->prev != NULL)
+	open_prevfile(TRUE);
+    else
+	return 1;
 
     unlink_opennode(tmp);
     delete_opennode(tmp);
 
-    shortcut_init(0);
+    shortcut_init(FALSE);
     display_main_list();
     return 0;
 }
@@ -1044,16 +1026,12 @@ char *get_full_path(const char *origpath)
     /* first, get the current directory, and tack a slash onto the end of
        it, unless it turns out to be "/", in which case leave it alone */
 
-#ifdef PATH_MAX
     d_here = getcwd(NULL, PATH_MAX + 1);
-#else
-    d_here = getcwd(NULL, 0);
-#endif
 
     if (d_here != NULL) {
 
 	align(&d_here);
-	if (strcmp(d_here, "/")) {
+	if (strcmp(d_here, "/") != 0) {
 	    d_here = charealloc(d_here, strlen(d_here) + 2);
 	    strcat(d_here, "/");
 	}
@@ -1114,18 +1092,14 @@ char *get_full_path(const char *origpath)
 
 		free(d_there);
 
-#ifdef PATH_MAX
 		d_there = getcwd(NULL, PATH_MAX + 1);
-#else
-		d_there = getcwd(NULL, 0);
-#endif
 
 		align(&d_there);
 		if (d_there != NULL) {
 
 		    /* add a slash to d_there, unless it's "/", in which
 		       case we don't need it */
-		    if (strcmp(d_there, "/")) {
+		    if (strcmp(d_there, "/") != 0) {
 			d_there = charealloc(d_there, strlen(d_there) + 2);
 			strcat(d_there, "/");
 		    }
@@ -1384,7 +1358,7 @@ int copy_file(FILE *inn, FILE *out)
     return retval;
 }
 
-/* Write a file out.  If tmp is nonzero, we set the umask to disallow
+/* Write a file out.  If tmp is FALSE, we set the umask to disallow
  * anyone else from accessing the file, we don't set the global variable
  * filename to its name, and we don't print out how many lines we wrote
  * on the statusbar.
@@ -1407,21 +1381,22 @@ int write_file(const char *name, int tmp, int append, int nonamechange)
     size_t lineswritten = 0;
     const filestruct *fileptr = fileage;
     int fd;
+	/* The file descriptor we use. */
     mode_t original_umask = 0;
 	/* Our umask, from when nano started. */
     int realexists;
-	/* The result of stat().  True if the file exists, false
+	/* The result of stat().  TRUE if the file exists, FALSE
 	 * otherwise.  If name is a link that points nowhere, realexists
-	 * is false. */
+	 * is FALSE. */
     struct stat st;
 	/* The status fields filled in by stat(). */
     int anyexists;
-	/* Result of lstat().  Same as realexists unless name is a
+	/* The result of lstat().  Same as realexists unless name is a
 	 * link. */
     struct stat lst;
 	/* The status fields filled in by lstat(). */
     char *realname;
-	/* name after ~ expansion. */
+	/* name after tilde expansion. */
     FILE *f;
 	/* The actual file, realname, we are writing to. */
     char *tempname = NULL;
@@ -1440,7 +1415,7 @@ int write_file(const char *name, int tmp, int append, int nonamechange)
 #ifndef DISABLE_OPERATINGDIR
     /* If we're writing a temporary file, we're probably going outside
      * the operating directory, so skip the operating directory test. */
-    if (!tmp && check_operating_dir(realname, 0) != 0) {
+    if (!tmp && check_operating_dir(realname, FALSE) != 0) {
 	statusbar(_("Can't write outside of %s"), operating_dir);
 	goto cleanup_and_exit;
     }
@@ -1453,7 +1428,8 @@ int write_file(const char *name, int tmp, int append, int nonamechange)
     /* If NOFOLLOW_SYMLINKS is set, it doesn't make sense to prepend or
      * append to a symlink.  Here we warn about the contradiction. */
     if (ISSET(NOFOLLOW_SYMLINKS) && anyexists && S_ISLNK(lst.st_mode)) {
-	statusbar(_("Cannot prepend or append to a symlink with --nofollow set"));
+	statusbar(
+		_("Cannot prepend or append to a symlink with --nofollow set"));
 	goto cleanup_and_exit;
     }
 
@@ -1547,8 +1523,7 @@ int write_file(const char *name, int tmp, int append, int nonamechange)
 		utime(backupname, &filetime) == -1) {
 	    free(backupname);
 	    if (copy_status == -1)
-		statusbar(_("Error reading %s: %s"), realname,
-			strerror(errno));
+		statusbar(_("Error reading %s: %s"), realname, strerror(errno));
 	    else
 		statusbar(_("Error writing %s: %s"), backupname,
 			strerror(errno));
@@ -1569,12 +1544,13 @@ int write_file(const char *name, int tmp, int append, int nonamechange)
 
     original_umask = umask(0);
     umask(original_umask);
+
     /* If we create a temp file, we don't let anyone else access it.  We
-     * create a temp file if tmp is nonzero or if we prepend. */
+     * create a temp file if tmp is TRUE or if we're prepending. */
     if (tmp || append == 2)
 	umask(S_IRWXG | S_IRWXO);
 
-    /* If we are prepending, copy the file to a temp file. */
+    /* If we're prepending, copy the file to a temp file. */
     if (append == 2) {
 	int fd_source;
 	FILE *f_source = NULL;
@@ -1615,13 +1591,13 @@ int write_file(const char *name, int tmp, int append, int nonamechange)
 	}
     }
 
-    /* Now open the file in place.  Use O_EXCL if tmp is nonzero.  This
-     * is now copied from joe, because wiggy says so *shrug*. */
+    /* Now open the file in place.  Use O_EXCL if tmp is TRUE.  This is
+     * now copied from joe, because wiggy says so *shrug*. */
     fd = open(realname, O_WRONLY | O_CREAT |
 	(append == 1 ? O_APPEND : (tmp ? O_EXCL : O_TRUNC)),
-	S_IRUSR | S_IWUSR);
+	S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
 
-    /* Put the umask back to the user's original value. */
+    /* Set the umask back to the user's original value. */
     umask(original_umask);
 
     /* First, just give up if we couldn't even open the file. */
@@ -1809,7 +1785,7 @@ int do_writeout(int exiting)
     currshortcut = writefile_list;
 #endif
 
-    if (exiting && filename[0] != '\0' && ISSET(TEMP_OPT)) {
+    if (exiting && filename[0] != '\0' && ISSET(TEMP_FILE)) {
 	i = write_file(filename, FALSE, 0, FALSE);
 	if (i == 1) {
 	    /* Write succeeded. */
@@ -1832,39 +1808,43 @@ int do_writeout(int exiting)
 	const char *formatstr, *backupstr;
 
 	if (ISSET(MAC_FILE))
-	   formatstr = _(" [Mac Format]");
+	   formatstr = N_(" [Mac Format]");
 	else if (ISSET(DOS_FILE))
-	   formatstr = _(" [DOS Format]");
+	   formatstr = N_(" [DOS Format]");
 	else
 	   formatstr = "";
 
 	if (ISSET(BACKUP_FILE))
-	   backupstr = _(" [Backup]");
+	   backupstr = N_(" [Backup]");
 	else
 	   backupstr = "";
 
 	/* Be nice to the translation folks. */
 	if (ISSET(MARK_ISSET) && !exiting) {
 	    if (append == 2)
-		msg = _("Prepend Selection to File");
+		msg = N_("Prepend Selection to File");
 	    else if (append == 1)
-		msg = _("Append Selection to File");
+		msg = N_("Append Selection to File");
 	    else
-		msg = _("Write Selection to File");
+		msg = N_("Write Selection to File");
 	} else
 #endif /* !NANO_SMALL */
 	if (append == 2)
-	    msg = _("File Name to Prepend to");
+	    msg = N_("File Name to Prepend to");
 	else if (append == 1)
-	    msg = _("File Name to Append to");
+	    msg = N_("File Name to Append to");
 	else
-	    msg = _("File Name to Write");
+	    msg = N_("File Name to Write");
 
-	i = statusq(TRUE, writefile_list,
+	/* If we're using restricted mode, the filename isn't blank,
+	 * and we're at the "Write File" prompt, disable tab
+	 * completion. */
+	i = statusq(!ISSET(RESTRICTED) || filename[0] == '\0',
+		writefile_list,
 #ifndef NANO_SMALL
-		ans, NULL, "%s%s%s", msg, formatstr, backupstr
+		ans, NULL, "%s%s%s", _(msg), formatstr, backupstr
 #else
-		filename, "%s", msg
+		filename, "%s", _(msg)
 #endif
 		);
 
@@ -1916,14 +1896,14 @@ int do_writeout(int exiting)
 #endif
 
 #ifdef NANO_EXTRA
-	if (exiting && !ISSET(TEMP_OPT) && !strcasecmp(answer, "zzy")
+	if (exiting && !ISSET(TEMP_FILE) && strcasecmp(answer, "zzy") == 0
 		&& !did_cred) {
 	    do_credits();
 	    did_cred = TRUE;
 	    return -1;
 	}
 #endif
-	if (append == 0 && strcmp(answer, filename)) {
+	if (append == 0 && strcmp(answer, filename) != 0) {
 	    struct stat st;
 
 	    if (!stat(answer, &st)) {
@@ -1940,7 +1920,7 @@ int do_writeout(int exiting)
 		&& (exiting || !ISSET(MARK_ISSET))
 #endif
 		) {
-		i = do_yesno(FALSE, _("Save file under DIFFERENT NAME ?" ));
+		i = do_yesno(FALSE, _("Save file under DIFFERENT NAME ? "));
 		if (i == 0 || i == -1)
 		    continue;
 	    }
@@ -1961,16 +1941,16 @@ int do_writeout(int exiting)
 	/* If we're not about to exit, update the current entry in
 	 * the open_files structure. */
 	if (!exiting)
-	    add_open_file(1);
+	    add_open_file(TRUE);
 #endif
 	display_main_list();
 	return i;
     } /* while (TRUE) */
 }
 
-int do_writeout_void(void)
+void do_writeout_void(void)
 {
-    return do_writeout(FALSE);
+    do_writeout(FALSE);
 }
 
 /* Return a malloc()ed string containing the actual directory, used
@@ -1995,7 +1975,7 @@ char *real_dir_from_tilde(const char *buf)
 	    do {
 		userdata = getpwent();
 	    } while (userdata != NULL &&
-			strncmp(userdata->pw_name, buf + 1, i - 1));
+			strncmp(userdata->pw_name, buf + 1, i - 1) != 0);
 	}
 	endpwent();
 
@@ -2015,7 +1995,7 @@ char *real_dir_from_tilde(const char *buf)
 /* Tack a slash onto the string we're completing if it's a directory.  We
  * assume there is room for one more character on the end of buf.  The
  * return value says whether buf is a directory. */
-int append_slash_if_dir(char *buf, int *lastwastab, int *place)
+int append_slash_if_dir(char *buf, bool *lastwastab, int *place)
 {
     char *dirptr = real_dir_from_tilde(buf);
     struct stat fileinfo;
@@ -2027,7 +2007,7 @@ int append_slash_if_dir(char *buf, int *lastwastab, int *place)
 	strncat(buf, "/", 1);
 	(*place)++;
 	/* now we start over again with # of tabs so far */
-	*lastwastab = 0;
+	*lastwastab = FALSE;
 	ret = 1;
     }
 
@@ -2067,7 +2047,7 @@ char **username_tab_completion(char *buf, int *num_matches)
 
     while ((userdata = getpwent()) != NULL) {
 
-	if (check_wildcard_match(userdata->pw_name, &buf[1]) == TRUE) {
+	if (check_wildcard_match(userdata->pw_name, &buf[1])) {
 
 	    /* Cool, found a match.  Add it to the list
 	     * This makes a lot more sense to me (Chris) this way...
@@ -2078,7 +2058,7 @@ char **username_tab_completion(char *buf, int *num_matches)
                directory, in which case just go to the next match */
 
 	    if (operating_dir != NULL) {
-		if (check_operating_dir(userdata->pw_dir, 1) != 0)
+		if (check_operating_dir(userdata->pw_dir, TRUE) != 0)
 		    continue;
 	    }
 #endif
@@ -2086,7 +2066,7 @@ char **username_tab_completion(char *buf, int *num_matches)
 	    matchline = charalloc(strlen(userdata->pw_name) + 2);
 	    sprintf(matchline, "~%s", userdata->pw_name);
 	    matches[*num_matches] = matchline;
-	    ++*num_matches;
+	    ++(*num_matches);
 
 	    /* If there's no more room, bail out */
 	    if (*num_matches == BUFSIZ)
@@ -2127,12 +2107,7 @@ char **cwd_tab_completion(char *buf, int *num_matches)
 
     } else {
 
-#ifdef PATH_MAX
 	if ((dirname = getcwd(NULL, PATH_MAX + 1)) == NULL)
-#else
-	/* The better, but apparently segfault-causing way */
-	if ((dirname = getcwd(NULL, 0)) == NULL)
-#endif /* PATH_MAX */
 	    return matches;
 	else
 	    tmp = buf;
@@ -2168,7 +2143,7 @@ char **cwd_tab_completion(char *buf, int *num_matches)
 	fprintf(stderr, "Comparing \'%s\'\n", next->d_name);
 #endif
 	/* See if this matches */
-	if (check_wildcard_match(next->d_name, tmp) == TRUE) {
+	if (check_wildcard_match(next->d_name, tmp)) {
 
 	    /* Cool, found a match.  Add it to the list
 	     * This makes a lot more sense to me (Chris) this way...
@@ -2186,7 +2161,7 @@ char **cwd_tab_completion(char *buf, int *num_matches)
 		strcpy(tmp2, dirname);
 		strcat(tmp2, "/");
 		strcat(tmp2, next->d_name);
-		if (check_operating_dir(tmp2, 1) != 0) {
+		if (check_operating_dir(tmp2, TRUE) != 0) {
 		    free(tmp2);
 		    continue;
 		}
@@ -2213,7 +2188,8 @@ char **cwd_tab_completion(char *buf, int *num_matches)
 
 /* This function now has an arg which refers to how much the statusbar
  * (place) should be advanced, i.e. the new cursor pos. */
-char *input_tab(char *buf, int place, int *lastwastab, int *newplace, int *list)
+char *input_tab(char *buf, int place, bool *lastwastab, int *newplace,
+	bool *list)
 {
     /* Do TAB completion */
     static int num_matches = 0, match_matches = 0;
@@ -2222,12 +2198,12 @@ char *input_tab(char *buf, int place, int *lastwastab, int *newplace, int *list)
     int longestname = 0, is_dir = 0;
     char *foo;
 
-    *list = 0;
+    *list = FALSE;
 
     if (*lastwastab == FALSE) {
 	char *tmp, *copyto, *matchbuf;
 
-	*lastwastab = 1;
+	*lastwastab = TRUE;
 
 	/* Make a local copy of the string -- up to the position of the
 	   cursor */
@@ -2297,7 +2273,7 @@ char *input_tab(char *buf, int place, int *lastwastab, int *newplace, int *list)
 	    } else
 		tmp = buf;
 
-	    if (!strcmp(tmp, matches[0]))
+	    if (strcmp(tmp, matches[0]) == 0)
 		is_dir = append_slash_if_dir(buf, lastwastab, newplace);
 
 	    if (is_dir != 0)
@@ -2414,14 +2390,14 @@ char *input_tab(char *buf, int place, int *lastwastab, int *newplace, int *list)
 	    }
 	    free(foo);
 	    wrefresh(edit);
-	    *list = 1;
+	    *list = TRUE;
 	} else
 	    beep();
     }
 
     /* Only refresh the edit window if we don't have a list of filename
        matches on it */
-    if (*list == 0)
+    if (*list == FALSE)
 	edit_refresh();
     curs_set(1);
     return buf;
@@ -2521,7 +2497,7 @@ char **browser_init(const char *path, int *longest, int *numents)
 
     *numents = 0;
     while ((next = readdir(dir)) != NULL) {
-	if (!strcmp(next->d_name, "."))
+	if (strcmp(next->d_name, ".") == 0)
 	   continue;
 	(*numents)++;
 	if (strlen(next->d_name) > *longest)
@@ -2532,12 +2508,12 @@ char **browser_init(const char *path, int *longest, int *numents)
 
     filelist = (char **)nmalloc(*numents * sizeof (char *));
 
-    if (!strcmp(path, "/"))
+    if (strcmp(path, "/") == 0)
 	path = "";
     path_len = strlen(path);
 
     while ((next = readdir(dir)) != NULL) {
-	if (!strcmp(next->d_name, "."))
+	if (strcmp(next->d_name, ".") == 0)
 	   continue;
 
 	filelist[i] = charalloc(strlen(next->d_name) + path_len + 2);
@@ -2571,7 +2547,7 @@ char *do_browser(const char *inpath)
     /* If path isn't the same as inpath, we are being passed a new
 	dir as an arg.  We free it here so it will be copied from 
 	inpath below */
-    if (path != NULL && strcmp(path, inpath)) {
+    if (path != NULL && strcmp(path, inpath) != 0) {
 	free(path);
 	path = NULL;
     }
@@ -2682,14 +2658,19 @@ char *do_browser(const char *inpath)
 	case NANO_HELP_KEY:
 	case NANO_HELP_FKEY:
 	case '?': /* Pico compatibility */
+#ifndef DISABLE_HELP
 	    do_help();
 	    curs_set(0);
+#else
+	    nano_disabled_msg();
+#endif
 	    break;
 	case NANO_ENTER_KEY:
 	case 'S': /* Pico compatibility */
 	case 's':
 	    /* You can't cd up from / */
-	    if (!strcmp(filelist[selected], "/..") && !strcmp(path, "/")) {
+	    if (strcmp(filelist[selected], "/..") == 0 &&
+		strcmp(path, "/") == 0) {
 		statusbar(_("Can't move up a directory"));
 		beep();
 		break;
@@ -2699,15 +2680,17 @@ char *do_browser(const char *inpath)
 	    /* Note: the selected file can be outside the operating
 	     * directory if it is .. or if it is a symlink to 
 	     * directory outside the operating directory. */
-	    if (check_operating_dir(filelist[selected], 0) != 0) {
-		statusbar(_("Can't go outside of %s in restricted mode"), operating_dir);
+	    if (check_operating_dir(filelist[selected], FALSE) != 0) {
+		statusbar(_("Can't go outside of %s in restricted mode"),
+			operating_dir);
 		beep();
 		break;
 	    }
 #endif
 
 	    if (stat(filelist[selected], &st) == -1) {
-		statusbar(_("Can't open \"%s\": %s"), filelist[selected], strerror(errno));
+		statusbar(_("Can't open \"%s\": %s"), filelist[selected],
+			strerror(errno));
 		beep();
 		break;
 	    }
@@ -2734,7 +2717,8 @@ char *do_browser(const char *inpath)
 
 	    if (!readable_dir(new_path)) {
 		/* We can't open this dir for some reason.  Complain */
-		statusbar(_("Can't open \"%s\": %s"), new_path, strerror(errno));
+		statusbar(_("Can't open \"%s\": %s"), new_path,
+			strerror(errno));
 		free(new_path);
 		break;
 	    }
@@ -2745,7 +2729,7 @@ char *do_browser(const char *inpath)
 	    path = new_path;
 	    return do_browser(path);
 
-	/* Goto a specific directory */
+	/* Go to a specific directory */
 	case NANO_GOTO_KEY:
 	case NANO_GOTO_FKEY:
 	case 'G': /* Pico compatibility */
@@ -2755,12 +2739,12 @@ char *do_browser(const char *inpath)
 #ifndef NANO_SMALL
 		NULL,
 #endif
-		_("Goto Directory"));
+		_("Go To Directory"));
 	    bottombars(browser_list);
 	    curs_set(0);
 
 	    if (j < 0) {
-		statusbar(_("Goto Cancelled"));
+		statusbar(_("Cancelled"));
 		break;
 	    }
 
@@ -2772,7 +2756,7 @@ char *do_browser(const char *inpath)
 	    }
 
 #ifndef DISABLE_OPERATINGDIR
-	    if (check_operating_dir(new_path, 0) != 0) {
+	    if (check_operating_dir(new_path, FALSE) != 0) {
 		statusbar(_("Can't go outside of %s in restricted mode"), operating_dir);
 		free(new_path);
 		break;
@@ -2915,7 +2899,7 @@ char *do_browse_from(const char *inpath)
 
 #ifndef DISABLE_OPERATINGDIR
     /* If the resulting path isn't in the operating directory, use that. */
-    if (check_operating_dir(path, 0) != 0)
+    if (check_operating_dir(path, FALSE) != 0)
 	path = mallocstrcpy(path, operating_dir);
 #endif
 
@@ -2929,116 +2913,109 @@ char *do_browse_from(const char *inpath)
 }
 #endif /* !DISABLE_BROWSER */
 
-#ifndef NANO_SMALL
-#ifdef ENABLE_NANORC
+#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+/* Return $HOME/.nano_history, or NULL if we can't find the homedir.
+ * The string is dynamically allocated, and should be freed. */
+char *histfilename(void)
+{
+    char *nanohist = NULL;
+
+    if (homedir != NULL) {
+	size_t homelen = strlen(homedir);
+
+	nanohist = charalloc(homelen + 15);
+	strcpy(nanohist, homedir);
+	strcpy(nanohist + homelen, "/.nano_history");
+    }
+    return nanohist;
+}
+
 void load_history(void)
 {
-    FILE *hist;
-    const struct passwd *userage = NULL;
-    static char *nanohist;
-    char *buf, *ptr;
-    char *homenv = getenv("HOME");
-    historyheadtype *history = &search_history;
+    char *nanohist = histfilename();
 
+    /* assume do_rcfile() has reported missing home dir */
+    if (nanohist != NULL) {
+	FILE *hist = fopen(nanohist, "r");
 
-    if (homenv != NULL) {
-        nanohist = charealloc(nanohist, strlen(homenv) + 15);
-        sprintf(nanohist, "%s/.nano_history", homenv);
-    } else {
-	userage = getpwuid(geteuid());
-	endpwent();
-        nanohist = charealloc(nanohist, strlen(userage->pw_dir) + 15);
-        sprintf(nanohist, "%s/.nano_history", userage->pw_dir);
-    }
-
-    /* assume do_rcfile has reported missing home dir */
-
-    if (homenv != NULL || userage != NULL) {
-	hist = fopen(nanohist, "r");
 	if (hist == NULL) {
-            if (errno != ENOENT) {
+	    if (errno != ENOENT) {
 		/* Don't save history when we quit. */
 		UNSET(HISTORYLOG);
-		rcfile_error(_("Unable to open ~/.nano_history file: %s"), strerror(errno));
+		rcfile_error(N_("Error reading %s: %s"), nanohist, strerror(errno));
+		fprintf(stderr, _("\nPress Return to continue starting nano\n"));
+		while (getchar() != '\n')
+		    ;
 	    }
-	    free(nanohist);
 	} else {
-	    buf = charalloc(1024);
-	    while (fgets(buf, 1023, hist) != 0) {
-		ptr = buf;
-		while (*ptr != '\n' && *ptr != '\0' && ptr < buf + 1023)
-		    ptr++;
-		*ptr = '\0';
-		if (strlen(buf))
-		    update_history(history, buf);
-		else
+	    historyheadtype *history = &search_history;
+	    char *line = NULL;
+	    size_t buflen = 0;
+	    ssize_t read;
+
+	    while ((read = getline(&line, &buflen, hist)) >= 0) {
+		if (read > 0 && line[read - 1] == '\n') {
+		    read--;
+		    line[read] = '\0';
+		}
+		if (read > 0) {
+		    unsunder(line, read);
+		    update_history(history, line);
+		} else
 		    history = &replace_history;
 	    }
 	    fclose(hist);
-	    free(buf);
-	    free(nanohist);
+	    free(line);
 	    UNSET(HISTORY_CHANGED);
 	}
+	free(nanohist);
     }
 }
 
+bool writehist(FILE *hist, historyheadtype *histhead)
+{
+    historytype *h;
+
+    /* write oldest first */
+    for (h = histhead->tail; h->prev != NULL; h = h->prev) {
+	size_t len = strlen(h->data);
+
+	sunder(h->data);
+	if (fwrite(h->data, sizeof(char), len, hist) < len ||
+		putc('\n', hist) == EOF)
+	    return FALSE;
+    }
+    return TRUE;
+}
+
 /* save histories to ~/.nano_history */
 void save_history(void)
 {
-    FILE *hist;
-    const struct passwd *userage = NULL;
-    char *nanohist = NULL;
-    char *homenv = getenv("HOME");
-    historytype *h;
+    char *nanohist;
 
     /* don't save unchanged or empty histories */
-    if (!((search_history.count || replace_history.count) &&
-			ISSET(HISTORY_CHANGED) && !ISSET(VIEW_MODE)))
+    if ((search_history.count == 0 && replace_history.count == 0) ||
+	!ISSET(HISTORY_CHANGED) || ISSET(VIEW_MODE))
 	return;
 
-    if (homenv != NULL) {
-	nanohist = charealloc(nanohist, strlen(homenv) + 15);
-	sprintf(nanohist, "%s/.nano_history", homenv);
-    } else {
-	userage = getpwuid(geteuid());
-	endpwent();
-	nanohist = charealloc(nanohist, strlen(userage->pw_dir) + 15);
-	sprintf(nanohist, "%s/.nano_history", userage->pw_dir);
-    }
+    nanohist = histfilename();
 
-    if (homenv != NULL || userage != NULL) {
-	hist = fopen(nanohist, "wb");
-	if (hist == NULL) {
-	    rcfile_msg(_("Unable to write ~/.nano_history file: %s"), strerror(errno));
-	} else {
+    if (nanohist != NULL) {
+	FILE *hist = fopen(nanohist, "wb");
+
+	if (hist == NULL)
+	    rcfile_error(N_("Error writing %s: %s"), nanohist, strerror(errno));
+	else {
 	    /* set rw only by owner for security ?? */
 	    chmod(nanohist, S_IRUSR | S_IWUSR);
-	    /* write oldest first */
-	    for (h = search_history.tail; h->prev; h = h->prev) {
-		h->data = charealloc(h->data, strlen(h->data) + 2);
-		strcat(h->data, "\n");
-		if (fputs(h->data, hist) == EOF) {
-		    rcfile_msg(_("Unable to write ~/.nano_history file: %s"), strerror(errno));
-		    goto come_from;
-		}
-	    }
-	    if (fputs("\n", hist) == EOF) {
-		    rcfile_msg(_("Unable to write ~/.nano_history file: %s"), strerror(errno));
-		    goto come_from;
-	    }
-	    for (h = replace_history.tail; h->prev; h = h->prev) {
-		h->data = charealloc(h->data, strlen(h->data) + 2);
-		strcat(h->data, "\n");
-		if (fputs(h->data, hist) == EOF) {
-		    rcfile_msg(_("Unable to write ~/.nano_history file: %s"), strerror(errno));
-		    goto come_from;
-		}
-	    }
-  come_from:
+
+	    if (!writehist(hist, &search_history) ||
+		    putc('\n', hist) == EOF ||
+		    !writehist(hist, &replace_history))
+		rcfile_error(N_("Error writing %s: %s"), nanohist, strerror(errno));
 	    fclose(hist);
 	}
 	free(nanohist);
     }
 }
-#endif /* ENABLE_NANORC */
-#endif /* !NANO_SMALL */
+#endif /* !NANO_SMALL && ENABLE_NANORC */
