@@ -1,4 +1,4 @@
-/* $Id: nano.h,v 1.41 2004/05/29 16:25:30 dolorous Exp $ */
+/* $Id: nano.h,v 1.51 2004/08/17 05:23:38 dolorous Exp $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -34,7 +34,7 @@
 #include <limits.h>
 #endif
 
-/* Macros for the flags int... */
+/* Macros for the flags long... */
 #define SET(bit) flags |= bit
 #define UNSET(bit) flags &= ~bit
 #define ISSET(bit) ((flags & bit) != 0)
@@ -76,6 +76,9 @@
 #  define _(string) (string)
 #  define P_(singular, plural, number) (number == 1 ? singular : plural)
 #endif
+#define gettext_noop(string) (string)
+#define N_(string) gettext_noop(string)
+	/* Mark a string that will be sent to gettext later. */
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -92,8 +95,8 @@
 # endif
 #endif
 
-/* If no isblank(), strcasecmp(), strncasecmp(), strcasestr(), or
- * strnlen(), use the versions we have. */
+/* If no isblank(), strcasecmp(), strncasecmp(), strcasestr(),
+ * strnlen(), getdelim(), or getline(), use the versions we have. */
 #ifndef HAVE_ISBLANK
 #define isblank is_blank_char
 #endif
@@ -114,6 +117,14 @@
 #define strnlen nstrnlen
 #endif
 
+#ifndef HAVE_GETDELIM
+#define getdelim ngetdelim
+#endif
+
+#ifndef HAVE_GETLINE
+#define getline ngetline
+#endif
+
 /* Assume ERR is defined as -1.  To avoid duplicate case values when
  * some key definitions are missing, we have to set all of these, and
  * all of the special sentinel values below, to different negative
@@ -160,10 +171,10 @@ typedef struct openfilestruct {
 				 * position. */
     int file_current_y;		/* Current file's y-coordinate
 				 * position. */
-    int file_flags;		/* Current file's flags: modification
+    long file_flags;		/* Current file's flags: modification
 				 * status (and marking status, if
 				 * available). */
-    int file_placewewant;	/* Current file's place we want. */
+    size_t file_placewewant;	/* Current file's place we want. */
     int file_totlines;		/* Current file's total number of
 				 * lines. */
     long file_totsize;		/* Current file's total size. */
@@ -179,7 +190,7 @@ typedef struct shortcut {
     int funcval;	/* Function key we want bound. */
     int miscval;	/* Other Meta key we want bound. */
     int viewok;		/* Is this function legal in view mode? */
-    int (*func) (void);	/* Function to call when we catch this key. */
+    void (*func)(void);	/* Function to call when we catch this key. */
     const char *desc;	/* Description, e.g. "Page Up". */
 #ifndef DISABLE_HELP
     const char *help;	/* Help file entry text. */
@@ -194,7 +205,7 @@ typedef struct toggle {
    const char *desc;	/* Description for when toggle is, uh, toggled,
 			 * e.g. "Cut to end"; we'll append Enabled or
 			 * Disabled. */
-   int flag;		/* What flag actually gets toggled. */
+   long flag;		/* What flag actually gets toggled. */
    struct toggle *next;
 } toggle;
 #endif /* !NANO_SMALL */
@@ -202,7 +213,7 @@ typedef struct toggle {
 #ifdef ENABLE_NANORC
 typedef struct rcoption {
    const char *name;
-   int flag;
+   long flag;
 } rcoption;
 #endif /* ENABLE_NANORC */
 
@@ -264,7 +275,7 @@ typedef struct historyheadtype {
 #define VIEW_MODE		(1<<9)
 #define USE_MOUSE		(1<<10)
 #define USE_REGEXP		(1<<11)
-#define TEMP_OPT		(1<<12)
+#define TEMP_FILE		(1<<12)
 #define CUT_TO_END		(1<<13)
 #define REVERSE_SEARCH		(1<<14)
 #define MULTIBUFFER		(1<<15)
@@ -362,6 +373,8 @@ typedef struct historyheadtype {
 #define NANO_HISTORY_KEY	-3
 
 /* Normal keys. */
+#define NANO_XON_KEY		NANO_CONTROL_Q
+#define NANO_XOFF_KEY		NANO_CONTROL_S
 #define NANO_INSERTFILE_KEY	NANO_CONTROL_R
 #define NANO_INSERTFILE_FKEY	KEY_F(5)
 #define NANO_EXIT_KEY		NANO_CONTROL_X
@@ -429,7 +442,7 @@ typedef struct historyheadtype {
 #define NANO_PREVWORD_KEY	NANO_ALT_SPACE
 #define NANO_PARABEGIN_KEY	NANO_CONTROL_W
 #define NANO_PARAEND_KEY	NANO_CONTROL_O
-#define NANO_FULLJUSTIFY_KEY	NANO_CONTROL_J
+#define NANO_FULLJUSTIFY_KEY	NANO_CONTROL_U
 #define NANO_VERBATIM_KEY	NANO_ALT_V
 
 #ifndef NANO_SMALL
@@ -457,12 +470,8 @@ typedef struct historyheadtype {
 
 #define MAIN_VISIBLE 12
 
-#define VIEW 1
-#define NOVIEW 0
-
-typedef enum {
-    JUSTIFY, BEGIN, END
-} justbegend;
+#define VIEW TRUE
+#define NOVIEW FALSE
 
 typedef enum {
     UP, DOWN
