@@ -1,4 +1,4 @@
-/* $Id: proto.h,v 1.57 2004/06/04 18:18:17 dolorous Exp $ */
+/* $Id: proto.h,v 1.81 2004/08/17 20:34:57 dolorous Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -22,28 +22,25 @@
 /* Externs. */
 
 #include <sys/stat.h>
-
+#include <sys/types.h>
 #ifdef HAVE_REGEX_H
 #include <regex.h>
 #endif
-
 #include "nano.h"
 
 #ifndef DISABLE_WRAPJUSTIFY
-extern int wrap_at;
+extern ssize_t wrap_at;
 #endif
 extern int editwinrows;
 extern int current_x, current_y, totlines;
-extern int placewewant;
+extern size_t placewewant;
 #ifndef NANO_SMALL
 extern int mark_beginx;
 #endif
 extern long totsize;
-extern int temp_opt;
-extern int flags;
-extern int tabsize;
+extern long flags;
+extern ssize_t tabsize;
 extern int search_last_line;
-extern int search_offscreen;
 extern int currslen;
 
 #if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
@@ -54,6 +51,13 @@ extern char *whitespace;
 extern char *punct;
 extern char *brackets;
 extern char *quotestr;
+#ifdef HAVE_REGEX_H
+extern regex_t quotereg;
+extern int quoterc;
+extern char *quoteerr;
+#else
+extern size_t quotelen;
+#endif
 #endif
 
 #ifndef NANO_SMALL
@@ -136,7 +140,11 @@ extern historyheadtype search_history;
 extern historyheadtype replace_history;
 #endif
 
-extern int curses_ended;
+extern bool curses_ended;
+
+#ifdef ENABLE_NANORC
+extern char *homedir;
+#endif
 
 /* Functions we want available. */
 
@@ -150,33 +158,34 @@ void update_color(void);
 /* Public functions in cut.c */
 void cutbuffer_reset(void);
 filestruct *get_cutbottom(void);
-void add_to_cutbuffer(filestruct *inptr, int allow_concat);
+void add_to_cutbuffer(filestruct *inptr, bool allow_concat);
 void cut_marked_segment(void);
-int do_cut_text(void);
-int do_uncut_text(void);
+void do_cut_text(void);
+void do_uncut_text(void);
 
 /* Public functions in files.c */
 void load_file(int update);
 void new_file(void);
 filestruct *read_line(char *buf, filestruct *prev, int *line1ins, size_t
 	len);
-int read_file(FILE *f, const char *filename, int quiet);
-int open_file(const char *filename, int insert, int quiet);
+void read_file(FILE *f, const char *filename, int quiet);
+bool open_file(const char *filename, int insert, int quiet);
 char *get_next_filename(const char *name);
-int do_insertfile(int loading_file);
-int do_insertfile_void(void);
+void do_insertfile(int loading_file);
+void do_insertfile_void(void);
 #ifdef ENABLE_MULTIBUFFER
 openfilestruct *make_new_opennode(openfilestruct *prevnode);
-void splice_opennode(openfilestruct *begin, openfilestruct *newnode, openfilestruct *end);
+void splice_opennode(openfilestruct *begin, openfilestruct *newnode,
+	openfilestruct *end);
 void unlink_opennode(const openfilestruct *fileptr);
 void delete_opennode(openfilestruct *fileptr);
 void free_openfilestruct(openfilestruct *src);
-int add_open_file(int update);
-int load_open_file(void);
-int open_prevfile(int closing_file);
-int open_prevfile_void(void);
-int open_nextfile(int closing_file);
-int open_nextfile_void(void);
+void add_open_file(int update);
+void load_open_file(void);
+void open_prevfile(int closing_file);
+void open_prevfile_void(void);
+void open_nextfile(int closing_file);
+void open_nextfile_void(void);
 int close_open_file(void);
 #endif
 #if !defined(DISABLE_SPELLER) || !defined(DISABLE_OPERATINGDIR)
@@ -193,18 +202,20 @@ int check_operating_dir(const char *currpath, int allow_tabcomp);
 #ifndef NANO_SMALL
 void init_backup_dir(void);
 #endif
+int copy_file(FILE *inn, FILE *out);
 int write_file(const char *name, int tmp, int append, int nonamechange);
 #ifndef NANO_SMALL
 int write_marked(const char *name, int tmp, int append);
 #endif
 int do_writeout(int exiting);
-int do_writeout_void(void);
+void do_writeout_void(void);
 char *real_dir_from_tilde(const char *buf);
 #ifndef DISABLE_TABCOMP
-int append_slash_if_dir(char *buf, int *lastwastab, int *place);
+int append_slash_if_dir(char *buf, bool *lastwastab, int *place);
 char **username_tab_completion(char *buf, int *num_matches);
 char **cwd_tab_completion(char *buf, int *num_matches);
-char *input_tab(char *buf, int place, int *lastwastab, int *newplace, int *list);
+char *input_tab(char *buf, int place, bool *lastwastab, int *newplace,
+	bool *list);
 #endif
 const char *tail(const char *foo);
 #ifndef DISABLE_BROWSER
@@ -216,41 +227,47 @@ char **browser_init(const char *path, int *longest, int *numents);
 char *do_browser(const char *inpath);
 char *do_browse_from(const char *inpath);
 #endif
+#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+char *histfilename(void);
+void load_history(void);
+bool writehist(FILE *hist, historyheadtype *histhead);
+void save_history(void);
+#endif
 
 /* Public functions in global.c */
-int length_of_list(const shortcut *s);
-void sc_init_one(shortcut **shortcutage, int key, const char *desc,
-#ifndef DISABLE_HELP
-	const char *help,
-#endif
-	int metaval, int funcval, int miscval, int view, int
-	(*func)(void));
+size_t length_of_list(const shortcut *s);
 #ifndef NANO_SMALL
-void toggle_init_one(int val, const char *desc, int flag);
+void toggle_init_one(int val, const char *desc, long flag);
 void toggle_init(void);
 #ifdef DEBUG
 void free_toggles(void);
 #endif
 #endif
-void free_shortcutage(shortcut **shortcutage);
+void sc_init_one(shortcut **shortcutage, int key, const char *desc,
+#ifndef DISABLE_HELP
+	const char *help,
+#endif
+	int metaval, int funcval, int miscval, int view, void
+	(*func)(void));
 void shortcut_init(int unjustify);
+void free_shortcutage(shortcut **shortcutage);
 #ifdef DEBUG
 void thanks_for_all_the_fish(void);
 #endif
 
 /* Public functions in move.c */
-int do_first_line(void);
-int do_last_line(void);
-int do_home(void);
-int do_end(void);
-int do_page_up(void);
-int do_page_down(void);
-int do_up(void);
-int do_down(void);
-int do_left(int allow_update);
-int do_left_void(void);
-int do_right(int allow_update);
-int do_right_void(void);
+void do_first_line(void);
+void do_last_line(void);
+void do_home(void);
+void do_end(void);
+void do_page_up(void);
+void do_page_down(void);
+void do_up(void);
+void do_down(void);
+void do_left(int allow_update);
+void do_left_void(void);
+void do_right(int allow_update);
+void do_right_void(void);
 
 /* Public functions in nano.c */
 void finish(void);
@@ -258,7 +275,7 @@ void die(const char *msg, ...);
 void die_save_file(const char *die_filename);
 void die_too_small(void);
 void print_view_warning(void);
-void global_init(int save_cutbuffer);
+void global_init(bool save_cutbuffer);
 void window_init(void);
 #ifndef DISABLE_MOUSE
 void mouse_init(void);
@@ -268,109 +285,100 @@ void help_init(void);
 #endif
 filestruct *make_new_node(filestruct *prevnode);
 filestruct *copy_node(const filestruct *src);
-void splice_node(filestruct *begin, filestruct *newnode, filestruct *end);
+void splice_node(filestruct *begin, filestruct *newnode, filestruct
+	*end);
 void unlink_node(const filestruct *fileptr);
 void delete_node(filestruct *fileptr);
 filestruct *copy_filestruct(const filestruct *src);
 void free_filestruct(filestruct *src);
 void renumber_all(void);
 void renumber(filestruct *fileptr);
-void print1opt(const char *shortflag, const char *longflag,
-		const char *desc);
+void print1opt(const char *shortflag, const char *longflag, const char
+	*desc);
 void usage(void);
 void version(void);
 int no_help(void);
-int nano_disabled_msg(void);
+void nano_disabled_msg(void);
 #ifndef NANO_SMALL
 RETSIGTYPE cancel_fork(int signal);
-int open_pipe(const char *command);
+bool open_pipe(const char *command);
 #endif
 #ifndef DISABLE_MOUSE
 void do_mouse(void);
 #endif
 void do_char(char ch);
-int do_verbatim_input(void);
-int do_backspace(void);
-int do_delete(void);
-int do_tab(void);
-int do_enter(void);
+void do_verbatim_input(void);
+void do_backspace(void);
+void do_delete(void);
+void do_tab(void);
+void do_enter(void);
 #ifndef NANO_SMALL
-int do_next_word(void);
-int do_prev_word(void);
-int do_mark(void);
+void do_next_word(void);
+void do_prev_word(void);
+void do_mark(void);
 #endif
 #ifndef DISABLE_WRAPPING
 void wrap_reset(void);
-int do_wrap(filestruct *inptr);
+bool do_wrap(filestruct *inptr);
 #endif
 #ifndef DISABLE_SPELLER
-int do_int_spell_fix(const char *word);
-const char *do_int_speller(char *tempfile_name);
+bool do_int_spell_fix(const char *word);
+const char *do_int_speller(const char *tempfile_name);
 const char *do_alt_speller(char *tempfile_name);
-int do_spell(void);
+void do_spell(void);
 #endif
-#if !defined(DISABLE_WRAPPING) && !defined(NANO_SMALL) || !defined(DISABLE_JUSTIFY)
+#if !defined(NANO_SMALL) || !defined(DISABLE_JUSTIFY)
 size_t indent_length(const char *line);
 #endif
 #ifndef DISABLE_JUSTIFY
 void justify_format(filestruct *line, size_t skip);
-#ifdef HAVE_REGEX_H
-size_t quote_length(const char *line, const regex_t *qreg);
-#else
 size_t quote_length(const char *line);
-#endif
-#ifdef HAVE_REGEX_H
-#  define IFREG(a, b) a, b
-#else
-#  define IFREG(a, b) a
-#endif
-int quotes_match(const char *a_line, size_t a_quote, IFREG(const char
-	*b_line, const regex_t *qreg));
-size_t indents_match(const char *a_line, size_t a_indent, const char
+bool quotes_match(const char *a_line, size_t a_quote, const char
+	*b_line);
+bool indents_match(const char *a_line, size_t a_indent, const char
 	*b_line, size_t b_indent);
+bool begpar(const filestruct *const foo);
+void do_para_begin(void);
+bool inpar(const char *str);
+void do_para_end(void);
 filestruct *backup_lines(filestruct *first_line, size_t par_len, size_t
 	quote_len);
-int breakable(const char *line, int goal);
-int break_line(const char *line, int goal, int force);
-int do_para_search(justbegend search_type, size_t *quote, size_t *par,
-	size_t *indent, int do_refresh);
-int do_para_begin(void);
-int do_para_end(void);
-int do_justify(int justify_all);
-int do_justify_void(void);
-int do_full_justify(void);
+bool breakable(const char *line, int goal);
+int break_line(const char *line, int goal, bool force);
+bool do_para_search(size_t *const quote, size_t *const par);
+void do_justify(bool full_justify);
+void do_justify_void(void);
+void do_full_justify(void);
 #endif /* !DISABLE_JUSTIFY */
-int do_exit(void);
+void do_exit(void);
 void signal_init(void);
 RETSIGTYPE handle_hupterm(int signal);
 RETSIGTYPE do_suspend(int signal);
 RETSIGTYPE do_cont(int signal);
 #ifndef NANO_SMALL
 void handle_sigwinch(int s);
-void allow_pending_sigwinch(int allow);
+void allow_pending_sigwinch(bool allow);
 #endif
 #ifndef NANO_SMALL
 void do_toggle(const toggle *which);
 #endif
-#if !defined(NANO_SMALL) || defined(USE_SLANG)
 void disable_signals(void);
-#endif
 #ifndef NANO_SMALL
 void enable_signals(void);
 #endif
 void disable_flow_control(void);
 void enable_flow_control(void);
+void terminal_init(void);
 
 /* Public functions in rcfile.c */
 #ifdef ENABLE_NANORC
 void rcfile_error(const char *msg, ...);
-void rcfile_msg(const char *msg, ...);
 char *parse_next_word(char *ptr);
 char *parse_argument(char *ptr);
 #ifdef ENABLE_COLOR
 int colortoint(const char *colorname, int *bright);
 char *parse_next_regex(char *ptr);
-int nregcomp(regex_t *preg, const char *regex, int flags);
+int nregcomp(regex_t *preg, const char *regex, int eflags);
 void parse_syntax(char *ptr);
 void parse_colors(char *ptr);
 #endif /* ENABLE_COLOR */
@@ -391,9 +399,9 @@ int is_whole_word(int curr_pos, const char *datastr, const char
 	*searchword);
 int findnextstr(int can_display_wrap, int wholeword, const filestruct
 	*begin, size_t beginx, const char *needle, int no_sameline);
-int do_search(void);
+void do_search(void);
 #ifndef NANO_SMALL
-int do_research(void);
+void do_research(void);
 #endif
 void replace_abort(void);
 #ifdef HAVE_REGEX_H
@@ -402,26 +410,24 @@ int replace_regexp(char *string, int create_flag);
 char *replace_line(const char *needle);
 int do_replace_loop(const char *needle, const filestruct *real_current,
 	size_t *real_current_x, int wholewords);
-int do_replace(void);
-int do_gotoline(int line, int save_pos);
-int do_gotoline_void(void);
+void do_replace(void);
+void do_gotoline(int line, bool save_pos);
+void do_gotoline_void(void);
 #if defined (ENABLE_MULTIBUFFER) || !defined (DISABLE_SPELLER)
-void do_gotopos(int line, int pos_x, int pos_y, int pos_placewewant);
+void do_gotopos(int line, int pos_x, int pos_y, size_t pos_pww);
 #endif
-int do_find_bracket(void);
+void do_find_bracket(void);
 #ifndef NANO_SMALL
 void history_init(void);
-historytype *find_node(historytype *h, char *s);
+historytype *find_node(historytype *h, const char *s);
 void remove_node(historytype *r);
 void insert_node(historytype *h, const char *s);
-void update_history(historyheadtype *h, char *s);
+void update_history(historyheadtype *h, const char *s);
 char *get_history_older(historyheadtype *h);
 char *get_history_newer(historyheadtype *h);
 char *get_history_completion(historyheadtype *h, char *s);
+#ifdef DEBUG
 void free_history(historyheadtype *h);
-#ifdef ENABLE_NANORC
-void load_history(void);
-void save_history(void);
 #endif
 #endif
 
@@ -438,6 +444,7 @@ int is_blank_char(int c);
 #endif
 int is_cntrl_char(int c);
 int num_of_digits(int n);
+bool parse_num(const char *str, ssize_t *val);
 void align(char **strp);
 void null_at(char **data, size_t index);
 void unsunder(char *str, size_t true_len);
@@ -460,6 +467,14 @@ const char *revstristr(const char *haystack, const char *needle, const
 #ifndef HAVE_STRNLEN
 size_t nstrnlen(const char *s, size_t maxlen);
 #endif
+#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+#ifndef HAVE_GETLINE
+ssize_t ngetline(char **lineptr, size_t *n, FILE *stream);
+#endif
+#ifndef HAVE_GETDELIM
+ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream);
+#endif
+#endif /* !NANO_SMALL && ENABLE_NANORC */
 const char *strstrwrapper(const char *haystack, const char *needle,
 	const char *start);
 void nperror(const char *s);
@@ -468,8 +483,8 @@ void *nrealloc(void *ptr, size_t howmuch);
 char *mallocstrcpy(char *dest, const char *src);
 void new_magicline(void);
 #ifndef NANO_SMALL
-void mark_order(const filestruct **top, size_t *top_x,
-		const filestruct **bot, size_t *bot_x);
+void mark_order(const filestruct **top, size_t *top_x, const filestruct
+	**bot, size_t *bot_x);
 #endif
 #ifndef DISABLE_TABCOMP
 int check_wildcard_match(const char *text, const char *pattern);
@@ -494,8 +509,8 @@ int get_control_kbinput(int kbinput);
 int get_escape_seq_kbinput(int *escape_seq, size_t es_len, int
 	*ignore_seq);
 int get_escape_seq_abcd(int kbinput);
-int *get_verbatim_kbinput(WINDOW *win, int *verbatim_kbinput, size_t
-	*verbatim_len, int allow_ascii);
+int *get_verbatim_kbinput(WINDOW *win, int *v_kbinput, size_t *v_len,
+	int allow_ascii);
 int get_untranslated_kbinput(int kbinput, size_t position, int
 	allow_ascii
 #ifndef NANO_SMALL
@@ -503,7 +518,7 @@ int get_untranslated_kbinput(int kbinput, size_t position, int
 #endif
 	);
 #ifndef DISABLE_MOUSE
-int get_mouseinput(int *mouse_x, int *mouse_y, int shortcut);
+int get_mouseinput(int *mouse_x, int *mouse_y, int allow_shortcuts);
 #endif
 size_t xplustabs(void);
 size_t actual_x(const char *str, size_t xplus);
@@ -522,7 +537,7 @@ int nanogetstr(int allowtabs, const char *buf, const char *def,
 #endif
 		const shortcut *s
 #ifndef DISABLE_TABCOMP
-		, int *list
+		, bool *list
 #endif
 		);
 void titlebar(const char *path);
@@ -538,10 +553,10 @@ void reset_cursor(void);
 void edit_add(const filestruct *fileptr, const char *converted, int
 	yval, size_t start);
 void update_line(const filestruct *fileptr, size_t index);
-int need_horizontal_update(int old_placewewant);
-int need_vertical_update(int old_placewewant);
+int need_horizontal_update(size_t old_pww);
+int need_vertical_update(size_t old_pww);
 void edit_scroll(updown direction, int nlines);
-void edit_redraw(const filestruct *old_current, int old_pww);
+void edit_redraw(const filestruct *old_current, size_t old_pww);
 void edit_refresh(void);
 void edit_update(filestruct *fileptr, topmidnone location);
 int statusq(int allowtabs, const shortcut *s, const char *def,
@@ -550,12 +565,14 @@ int statusq(int allowtabs, const shortcut *s, const char *def,
 #endif
 		const char *msg, ...);
 int do_yesno(int all, const char *msg);
-int total_refresh(void);
+void total_refresh(void);
 void display_main_list(void);
-int do_cursorpos(int constant);
-int do_cursorpos_void(void);
-int line_len(const char *ptr);
-int do_help(void);
+void do_cursorpos(int constant);
+void do_cursorpos_void(void);
+#ifndef DISABLE_HELP
+int help_line_len(const char *ptr);
+void do_help(void);
+#endif
 void do_replace_highlight(int highlight_flag, const char *word);
 #ifdef DEBUG
 void dump_buffer(const filestruct *inptr);
