@@ -1,4 +1,4 @@
-/* $Id: rcfile.c,v 1.20 2004/05/30 04:19:27 dolorous Exp $ */
+/* $Id: rcfile.c,v 1.32 2004/08/17 05:23:38 dolorous Exp $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -27,7 +27,6 @@
 #include <stdio.h>
 #include <errno.h>
 #include <unistd.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <pwd.h>
@@ -92,7 +91,7 @@ const static rcoption rcopts[] = {
 #endif
     {"suspend", SUSPEND},
     {"tabsize", 0},
-    {"tempfile", TEMP_OPT},
+    {"tempfile", TEMP_FILE},
     {"view", VIEW_MODE},
 #ifndef NANO_SMALL
     {"whitespace", 0},
@@ -100,9 +99,9 @@ const static rcoption rcopts[] = {
     {NULL, 0}
 };
 
-static int errors = 0;
+static bool errors = FALSE;
 static int lineno = 0;
-static char *nanorc;
+static const char *nanorc;
 
 /* We have an error in some part of the rcfile; put it on stderr and
    make the user hit return to continue starting up nano. */
@@ -111,29 +110,15 @@ void rcfile_error(const char *msg, ...)
     va_list ap;
 
     fprintf(stderr, "\n");
-    if (lineno > 0)
+    if (lineno > 0) {
+	errors = TRUE;
 	fprintf(stderr, _("Error in %s on line %d: "), nanorc, lineno);
-
-    va_start(ap, msg);
-    vfprintf(stderr, msg, ap);
-    va_end(ap);
-    fprintf(stderr, _("\nPress return to continue starting nano\n"));
-
-    while (getchar() != '\n');
-}
-
-/* Just print the error (one of many, perhaps) but don't abort, yet. */
-void rcfile_msg(const char *msg, ...)
-{
-    va_list ap;
-
-    if (!errors) {
-	errors = 1;
-	fprintf(stderr, "\n");
     }
+
     va_start(ap, msg);
-    vfprintf(stderr, msg, ap);
+    vfprintf(stderr, _(msg), ap);
     va_end(ap);
+
     fprintf(stderr, "\n");
 }
 
@@ -183,7 +168,7 @@ char *parse_argument(char *ptr)
 	    ptr = NULL;
 	else
 	    *ptr++ = '\0';
-	rcfile_error(_("Argument %s has unterminated \""), ptr_bak);
+	rcfile_error(N_("Argument %s has unterminated \""), ptr_bak);
     } else {
 	*last_quote = '\0';
 	ptr = last_quote + 1;
@@ -203,33 +188,33 @@ int colortoint(const char *colorname, int *bright)
     if (colorname == NULL)
 	return -1;
 
-    if (!strncasecmp(colorname, "bright", 6)) {
+    if (strncasecmp(colorname, "bright", 6) == 0) {
 	*bright = 1;
 	colorname += 6;
     }
 
-    if (!strcasecmp(colorname, "green"))
+    if (strcasecmp(colorname, "green") == 0)
 	mcolor = COLOR_GREEN;
-    else if (!strcasecmp(colorname, "red"))
+    else if (strcasecmp(colorname, "red") == 0)
 	mcolor = COLOR_RED;
-    else if (!strcasecmp(colorname, "blue"))
+    else if (strcasecmp(colorname, "blue") == 0)
 	mcolor = COLOR_BLUE;
-    else if (!strcasecmp(colorname, "white"))
+    else if (strcasecmp(colorname, "white") == 0)
 	mcolor = COLOR_WHITE;
-    else if (!strcasecmp(colorname, "yellow"))
+    else if (strcasecmp(colorname, "yellow") == 0)
 	mcolor = COLOR_YELLOW;
-    else if (!strcasecmp(colorname, "cyan"))
+    else if (strcasecmp(colorname, "cyan") == 0)
 	mcolor = COLOR_CYAN;
-    else if (!strcasecmp(colorname, "magenta"))
+    else if (strcasecmp(colorname, "magenta") == 0)
 	mcolor = COLOR_MAGENTA;
-    else if (!strcasecmp(colorname, "black"))
+    else if (strcasecmp(colorname, "black") == 0)
 	mcolor = COLOR_BLACK;
     else {
-	rcfile_error(_("Color %s not understood.\n"
-			"Valid colors are \"green\", \"red\", \"blue\", \n"
-			"\"white\", \"yellow\", \"cyan\", \"magenta\" and \n"
-			"\"black\", with the optional prefix \"bright\" \n"
-			"for foreground colors.\n"), colorname);
+	rcfile_error(N_("Color %s not understood.\n"
+		"Valid colors are \"green\", \"red\", \"blue\", \n"
+		"\"white\", \"yellow\", \"cyan\", \"magenta\" and \n"
+		"\"black\", with the optional prefix \"bright\" \n"
+		"for foreground colors."), colorname);
 	mcolor = -1;
     }
     return mcolor;
@@ -253,18 +238,18 @@ char *parse_next_regex(char *ptr)
     return ptr;
 }
 
-/* Compile the regular expression regex to preg.  Returns FALSE on success,
-   TRUE if the expression is invalid. */
-int nregcomp(regex_t *preg, const char *regex, int flags)
+/* Compile the regular expression regex to preg.  Returns FALSE on
+ * success, or TRUE if the expression is invalid. */
+int nregcomp(regex_t *preg, const char *regex, int eflags)
 {
-    int rc = regcomp(preg, regex, REG_EXTENDED | flags);
+    int rc = regcomp(preg, regex, REG_EXTENDED | eflags);
 
     if (rc != 0) {
 	size_t len = regerror(rc, preg, NULL, 0);
 	char *str = charalloc(len);
 
 	regerror(rc, preg, str, len);
-	rcfile_error(_("Bad regex \"%s\": %s"), regex, str);
+	rcfile_error(N_("Bad regex \"%s\": %s"), regex, str);
 	free(str);
     }
     return rc != 0;
@@ -284,7 +269,7 @@ void parse_syntax(char *ptr)
 	return;
 
     if (*ptr != '"') {
-	rcfile_error(_("Regex strings must begin and end with a \" character\n"));
+	rcfile_error(N_("Regex strings must begin and end with a \" character"));
 	return;
     }
     ptr++;
@@ -293,7 +278,7 @@ void parse_syntax(char *ptr)
     ptr = parse_next_regex(ptr);
 
     if (ptr == NULL) {
-	rcfile_error(_("Missing syntax name"));
+	rcfile_error(N_("Missing syntax name"));
 	return;
     }
 
@@ -362,7 +347,7 @@ void parse_colors(char *ptr)
     ptr = parse_next_word(ptr);
 
     if (ptr == NULL) {
-	rcfile_error(_("Missing color name"));
+	rcfile_error(N_("Missing color name"));
 	return;
     }
 
@@ -370,8 +355,8 @@ void parse_colors(char *ptr)
 	char *bgcolorname;
 	strtok(fgstr, ",");
 	bgcolorname = strtok(NULL, ",");
-	if (!strncasecmp(bgcolorname, "bright", 6)) {
-	    rcfile_error(_("Background color %s cannot be bright"), bgcolorname);
+	if (strncasecmp(bgcolorname, "bright", 6) == 0) {
+	    rcfile_error(N_("Background color %s cannot be bright"), bgcolorname);
 	    return;
 	}
 	bg = colortoint(bgcolorname, &bright);
@@ -385,7 +370,7 @@ void parse_colors(char *ptr)
 	return;
 
     if (syntaxes == NULL) {
-	rcfile_error(_("Cannot add a color directive without a syntax line"));
+	rcfile_error(N_("Cannot add a color directive without a syntax line"));
 	return;
     }
 
@@ -408,13 +393,13 @@ void parse_colors(char *ptr)
 	if (*ptr == '\n' || *ptr == '\0')
 	    break;
 
-	if (!strncasecmp(ptr, "start=", 6)) {
+	if (strncasecmp(ptr, "start=", 6) == 0) {
 	    ptr += 6;
 	    expectend = 1;
 	}
 
 	if (*ptr != '"') {
-	    rcfile_error(_("Regex strings must begin and end with a \" character\n"));
+	    rcfile_error(N_("Regex strings must begin and end with a \" character"));
 	    ptr = parse_next_regex(ptr);
 	    continue;
 	}
@@ -436,8 +421,7 @@ void parse_colors(char *ptr)
 	    if (tmpsyntax->color == NULL) {
 		tmpsyntax->color = newcolor;
 #ifdef DEBUG
-		fprintf(stderr, "Starting a new colorstring for fg %d bg %d\n",
-			fg, bg);
+		fprintf(stderr, "Starting a new colorstring for fg %d bg %d\n", fg, bg);
 #endif
 	    } else {
 		for (tmpcolor = tmpsyntax->color; tmpcolor->next != NULL;
@@ -451,17 +435,15 @@ void parse_colors(char *ptr)
 	}
 
 	if (expectend) {
-	    if (ptr == NULL || strncasecmp(ptr, "end=", 4)) {
-		rcfile_error(_
-			     ("\"start=\" requires a corresponding \"end=\""));
+	    if (ptr == NULL || strncasecmp(ptr, "end=", 4) != 0) {
+		rcfile_error(N_("\"start=\" requires a corresponding \"end=\""));
 		return;
 	    }
 
 	    ptr += 4;
 
 	    if (*ptr != '"') {
-		rcfile_error(_
-			     ("Regex strings must begin and end with a \" character\n"));
+		rcfile_error(N_("Regex strings must begin and end with a \" character"));
 		continue;
 	    }
 	    ptr++;
@@ -488,7 +470,7 @@ void parse_colors(char *ptr)
 void parse_rcfile(FILE *rcstream)
 {
     char *buf, *ptr, *keyword, *option;
-    int set = 0, i, j;
+    int set = 0, i;
 
     buf = charalloc(1024);
     while (fgets(buf, 1023, rcstream) != 0) {
@@ -514,18 +496,18 @@ void parse_rcfile(FILE *rcstream)
 	    continue;
 
 	/* Else try to parse the keyword */
-	if (!strcasecmp(keyword, "set"))
+	if (strcasecmp(keyword, "set") == 0)
 	    set = 1;
-	else if (!strcasecmp(keyword, "unset"))
+	else if (strcasecmp(keyword, "unset") == 0)
 	    set = -1;
 #ifdef ENABLE_COLOR
-	else if (!strcasecmp(keyword, "syntax"))
+	else if (strcasecmp(keyword, "syntax") == 0)
 	    parse_syntax(ptr);
-	else if (!strcasecmp(keyword, "color"))
+	else if (strcasecmp(keyword, "color") == 0)
 	    parse_colors(ptr);
 #endif				/* ENABLE_COLOR */
 	else {
-	    rcfile_msg(_("Command %s not understood"), keyword);
+	    rcfile_error(N_("Command %s not understood"), keyword);
 	    continue;
 	}
 
@@ -535,38 +517,36 @@ void parse_rcfile(FILE *rcstream)
 
 	if (set != 0) {
 	    for (i = 0; rcopts[i].name != NULL; i++) {
-		if (!strcasecmp(option, rcopts[i].name)) {
+		if (strcasecmp(option, rcopts[i].name) == 0) {
 #ifdef DEBUG
 		    fprintf(stderr, "%s: Parsing option %s\n", 
 			    "parse_rcfile()", rcopts[i].name);
 #endif
 		    if (set == 1) {
-			if (!strcasecmp(rcopts[i].name, "tabsize")
+			if (strcasecmp(rcopts[i].name, "tabsize") == 0
 #ifndef DISABLE_OPERATINGDIR
-				|| !strcasecmp(rcopts[i].name, "operatingdir")
+				|| strcasecmp(rcopts[i].name, "operatingdir") == 0
 #endif
 #ifndef DISABLE_WRAPJUSTIFY
-				|| !strcasecmp(rcopts[i].name, "fill")
+				|| strcasecmp(rcopts[i].name, "fill") == 0
 #endif
 #ifndef NANO_SMALL
-				|| !strcasecmp(rcopts[i].name, "whitespace")
+				|| strcasecmp(rcopts[i].name, "whitespace") == 0
 #endif
 #ifndef DISABLE_JUSTIFY
-				|| !strcasecmp(rcopts[i].name, "punct")
-				|| !strcasecmp(rcopts[i].name, "brackets")
-				|| !strcasecmp(rcopts[i].name, "quotestr")
+				|| strcasecmp(rcopts[i].name, "punct") == 0
+				|| strcasecmp(rcopts[i].name, "brackets") == 0
+				|| strcasecmp(rcopts[i].name, "quotestr") == 0
 #endif
 #ifndef NANO_SMALL
-			        || !strcasecmp(rcopts[i].name, "backupdir")
+			        || strcasecmp(rcopts[i].name, "backupdir") == 0
 #endif
 #ifndef DISABLE_SPELLER
-				|| !strcasecmp(rcopts[i].name, "speller")
+				|| strcasecmp(rcopts[i].name, "speller") == 0
 #endif
 				) {
 			    if (*ptr == '\n' || *ptr == '\0') {
-				rcfile_error(_
-					     ("Option %s requires an argument"),
-					     rcopts[i].name);
+				rcfile_error(N_("Option %s requires an argument"), rcopts[i].name);
 				continue;
 			    }
 			    option = ptr;
@@ -577,79 +557,63 @@ void parse_rcfile(FILE *rcstream)
 			    fprintf(stderr, "option = %s\n", option);
 #endif
 #ifndef DISABLE_OPERATINGDIR
-			    if (!strcasecmp(rcopts[i].name, "operatingdir"))
+			    if (strcasecmp(rcopts[i].name, "operatingdir") == 0)
 				operating_dir = mallocstrcpy(NULL, option);
 			    else
 #endif
 #ifndef DISABLE_WRAPJUSTIFY
-			    if (!strcasecmp(rcopts[i].name, "fill")) {
-				char *first_error;
-
-				/* Using strtol() instead of atoi() lets
-				 * us accept 0 while checking other
-				 * errors. */
-				j = (int)strtol(option, &first_error, 10);
-				if (errno == ERANGE || *option == '\0' || *first_error != '\0')
-				    rcfile_error(_("Requested fill size %d invalid"),
-						 j);
-				else
-				    wrap_at = j;
+			    if (strcasecmp(rcopts[i].name, "fill") == 0) {
+				if (!parse_num(option, &wrap_at)) {
+				    rcfile_error(N_("Requested fill size %s invalid"), option);
+				    wrap_at = -CHARS_FROM_EOL;
+				}
 			    } else
 #endif
 #ifndef NANO_SMALL
-			    if (!strcasecmp(rcopts[i].name, "whitespace")) {
+			    if (strcasecmp(rcopts[i].name, "whitespace") == 0) {
 				size_t ws_len;
 				whitespace = mallocstrcpy(NULL, option);
 				ws_len = strlen(whitespace);
 				if (ws_len != 2 || (ws_len == 2 && (is_cntrl_char(whitespace[0]) || is_cntrl_char(whitespace[1])))) {
-				    rcfile_error(_("Two non-control characters required"));
+				    rcfile_error(N_("Two non-control characters required"));
 				    free(whitespace);
 				    whitespace = NULL;
-				} else
-				    SET(WHITESPACE_DISPLAY);
+				}
 			    } else
 #endif
 #ifndef DISABLE_JUSTIFY
-			    if (!strcasecmp(rcopts[i].name, "punct")) {
+			    if (strcasecmp(rcopts[i].name, "punct") == 0) {
 				punct = mallocstrcpy(NULL, option);
 				if (strchr(punct, '\t') != NULL || strchr(punct, ' ') != NULL) {
-				    rcfile_error(_("Non-tab and non-space characters required"));
+				    rcfile_error(N_("Non-tab and non-space characters required"));
 				    free(punct);
 				    punct = NULL;
 				}
-			    } else if (!strcasecmp(rcopts[i].name, "brackets")) {
+			    } else if (strcasecmp(rcopts[i].name, "brackets") == 0) {
 				brackets = mallocstrcpy(NULL, option);
 				if (strchr(brackets, '\t') != NULL || strchr(brackets, ' ') != NULL) {
-				    rcfile_error(_("Non-tab and non-space characters required"));
+				    rcfile_error(N_("Non-tab and non-space characters required"));
 				    free(brackets);
 				    brackets = NULL;
 				}
-			    } else if (!strcasecmp(rcopts[i].name, "quotestr"))
+			    } else if (strcasecmp(rcopts[i].name, "quotestr") == 0)
 				quotestr = mallocstrcpy(NULL, option);
 			    else
 #endif
 #ifndef NANO_SMALL
-			    if (!strcasecmp(rcopts[i].name, "backupdir"))
+			    if (strcasecmp(rcopts[i].name, "backupdir") == 0)
 				backup_dir = mallocstrcpy(NULL, option);
 			    else
 #endif
 #ifndef DISABLE_SPELLER
-			    if (!strcasecmp(rcopts[i].name, "speller"))
+			    if (strcasecmp(rcopts[i].name, "speller") == 0)
 				alt_speller = mallocstrcpy(NULL, option);
 			    else
 #endif
-			    if (!strcasecmp(rcopts[i].name, "tabsize")) {
-				char *first_error;
-
-				/* Using strtol instead of atoi lets us
-				 * accept 0 while checking other
-				 * errors. */
-				j = (int)strtol(option, &first_error, 10);
-				if (errno == ERANGE || *option == '\0' || *first_error != '\0')
-				    rcfile_error(_("Requested tab size %d invalid"),
-						 j);
-				else
-				    tabsize = j;
+			    if (strcasecmp(rcopts[i].name, "tabsize") == 0) {
+				if (!parse_num(option, &tabsize) || tabsize <= 0)
+				    rcfile_error(N_("Requested tab size %s invalid"), option);
+				    tabsize = -1;
 			    }
 			} else
 			    SET(rcopts[i].flag);
@@ -669,8 +633,13 @@ void parse_rcfile(FILE *rcstream)
 	}
     }
     free(buf);
-    if (errors)
-	rcfile_error(_("Errors found in .nanorc file"));
+
+    if (errors) {
+	errors = FALSE;
+	fprintf(stderr, _("\nPress Return to continue starting nano\n"));
+	while (getchar() != '\n')
+	    ;
+    }
 
     return;
 }
@@ -679,16 +648,13 @@ void parse_rcfile(FILE *rcstream)
 void do_rcfile(void)
 {
     FILE *rcstream;
-    const struct passwd *userage;
-    uid_t euid = geteuid();
-    char *homenv = getenv("HOME");
 
 #ifdef SYSCONFDIR
     assert(sizeof(SYSCONFDIR) == strlen(SYSCONFDIR) + 1);
-    nanorc = charalloc(sizeof(SYSCONFDIR) + 7);
-    sprintf(nanorc, "%s/nanorc", SYSCONFDIR);
+    nanorc = SYSCONFDIR "/nanorc";
     /* Try to open system nanorc */
-    if ((rcstream = fopen(nanorc, "r")) != NULL) {
+    rcstream = fopen(nanorc, "r");
+    if (rcstream != NULL) {
 	/* Parse it! */
 	parse_rcfile(rcstream);
 	fclose(rcstream);
@@ -697,47 +663,51 @@ void do_rcfile(void)
 
     lineno = 0;
 
-    /* Rely on $HOME, fall back on getpwuid() */
-    if (homenv != NULL) {
-	nanorc = charealloc(nanorc, strlen(homenv) + 10);
-	sprintf(nanorc, "%s/.nanorc", homenv);
-    } else {
-	userage = getpwuid(euid);
-	endpwent();
+    {
+	const char *homenv = getenv("HOME");
 
-	if (userage == NULL) {
-	    rcfile_error(_("I can't find my home directory!  Wah!"));
-	    SET(NO_RCFILE);
-	} else {
-	    nanorc = charealloc(nanorc, strlen(userage->pw_dir) + 9);
-	    sprintf(nanorc, "%s/.nanorc", userage->pw_dir);
+	/* Rely on $HOME, fall back on getpwuid() */
+	if (homenv == NULL) {
+	    const struct passwd *userage = getpwuid(geteuid());
 
+	    if (userage != NULL)
+		homenv = userage->pw_dir;
 	}
+	homedir = mallocstrcpy(NULL, homenv);
     }
 
-    if (!ISSET(NO_RCFILE)) {
+    if (homedir == NULL) {
+	rcfile_error(N_("I can't find my home directory!  Wah!"));
+	SET(NO_RCFILE);
+    } else {
+	size_t homelen = strlen(homedir);
+	char *nanorcf = charalloc(homelen + 9);
+
+	nanorc = nanorcf;
+	strcpy(nanorcf, homedir);
+	strcpy(nanorcf + homelen, "/.nanorc");
 
 #if defined(DISABLE_ROOTWRAP) && !defined(DISABLE_WRAPPING)
     /* If we've already read SYSCONFDIR/nanorc (if it's there), we're
        root, and --disable-wrapping-as-root is used, turn wrapping off */
-	if (euid == NANO_ROOT_UID)
+	if (geteuid() == NANO_ROOT_UID)
 	    SET(NO_WRAP);
 #endif
-	if ((rcstream = fopen(nanorc, "r")) == NULL) {
+	rcstream = fopen(nanorc, "r");
+	if (rcstream == NULL) {
 	    /* Don't complain about the file not existing */
 	    if (errno != ENOENT) {
-		rcfile_error(_("Unable to open ~/.nanorc file, %s"),
-			strerror(errno));
+		rcfile_error(N_("Error reading %s: %s"), nanorc, strerror(errno));
 		SET(NO_RCFILE);
 	    }
 	} else {
 	    parse_rcfile(rcstream);
 	    fclose(rcstream);
 	}
+	free(nanorcf);
     }
     lineno = 0;
 
-    free(nanorc);
 #ifdef ENABLE_COLOR
     set_colorpairs();
 #endif
