@@ -1,4 +1,4 @@
-/* $Id: search.c,v 1.24 2004/05/30 03:19:52 dolorous Exp $ */
+/* $Id: search.c,v 1.35 2004/08/11 05:13:08 dolorous Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -238,11 +238,9 @@ int search_init(int replacing)
 #ifndef NANO_SMALL
 	    search_history.current = search_history.next;
 #endif
-	    i = (int)strtol(answer, &buf, 10);	/* Just testing answer here. */
-	    if (!(errno == ERANGE || *answer == '\0' || *buf != '\0'))
-		do_gotoline(-1, 0);
-	    else
-		do_gotoline_void();
+	   /* If answer parses as an integer, put it up on the
+	    * statusbar. */
+	    do_gotoline(parse_num(answer, NULL) ? -1 : 0, FALSE);
 	    /* Fall through. */
 	default:
 	    return -1;
@@ -359,9 +357,10 @@ int findnextstr(int can_display_wrap, int wholeword, const filestruct
 }
 
 /* Search for a string. */
-int do_search(void)
+void do_search(void)
 {
-    int old_pww = placewewant, i, fileptr_x = current_x, didfind;
+    size_t old_pww = placewewant, fileptr_x = current_x;
+    int i, didfind;
     filestruct *fileptr = current;
 
 #ifndef DISABLE_WRAPPING
@@ -380,7 +379,7 @@ int do_search(void)
 #endif
 
     if (i != 0)
-	return 0;
+	return;
 
     /* If answer is now "", copy last_search into answer. */
     if (answer[0] == '\0')
@@ -407,8 +406,10 @@ int do_search(void)
 	 * that we find one only once per line.  We should only end up
 	 * back at the same position if the string isn't found again, in
 	 * which case it's the only occurrence. */
-	if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp, last_search)) {
-	    didfind = findnextstr(TRUE, FALSE, current, current_x, answer, TRUE);
+	if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp,
+		last_search)) {
+	    didfind = findnextstr(TRUE, FALSE, current, current_x,
+		answer, TRUE);
 	    if (fileptr == current && fileptr_x == current_x && !didfind)
 		statusbar(_("This is the only occurrence"));
 	} else {
@@ -422,15 +423,14 @@ int do_search(void)
     placewewant = xplustabs();
     edit_redraw(fileptr, old_pww);
     search_abort();
-
-    return 1;
 }
 
 #ifndef NANO_SMALL
 /* Search for the next string without prompting. */
-int do_research(void)
+void do_research(void)
 {
-    int old_pww = placewewant, fileptr_x = current_x, didfind;
+    size_t old_pww = placewewant, fileptr_x = current_x;
+    int didfind;
     filestruct *fileptr = current;
 
 #ifndef DISABLE_WRAPPING
@@ -443,11 +443,12 @@ int do_research(void)
 #ifdef HAVE_REGEX_H
 	/* Since answer is "", use last_search! */
 	if (ISSET(USE_REGEXP) && regexp_init(last_search) == 0)
-	    return -1;
+	    return;
 #endif
 
 	search_last_line = FALSE;
-	didfind = findnextstr(TRUE, FALSE, current, current_x, last_search, FALSE);
+	didfind = findnextstr(TRUE, FALSE, current, current_x,
+		last_search, FALSE);
 
 	/* Check to see if there's only one occurrence of the string and
 	 * we're on it now. */
@@ -458,8 +459,10 @@ int do_research(void)
 	     * "^$"), so that we find one only once per line.  We should
 	     * only end up back at the same position if the string isn't
 	     * found again, in which case it's the only occurrence. */
-	    if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp, last_search)) {
-		didfind = findnextstr(TRUE, FALSE, current, current_x, answer, TRUE);
+	    if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp,
+		last_search)) {
+		didfind = findnextstr(TRUE, FALSE, current, current_x,
+			answer, TRUE);
 		if (fileptr == current && fileptr_x == current_x && !didfind)
 		    statusbar(_("This is the only occurrence"));
 	    } else {
@@ -475,8 +478,6 @@ int do_research(void)
     placewewant = xplustabs();
     edit_redraw(fileptr, old_pww);
     search_abort();
-
-    return 1;
 }
 #endif
 
@@ -587,8 +588,8 @@ char *replace_line(const char *needle)
 int do_replace_loop(const char *needle, const filestruct *real_current,
 	size_t *real_current_x, int wholewords)
 {
-    int old_pww = placewewant, replaceall = 0, numreplaced = -1;
-    size_t current_x_save = current_x;
+    int replaceall = FALSE, numreplaced = -1;
+    size_t old_pww = placewewant, current_x_save = current_x;
     const filestruct *current_save = current;
 #ifdef HAVE_REGEX_H
     /* The starting-line match and bol/eol regex flags. */
@@ -634,8 +635,11 @@ int do_replace_loop(const char *needle, const filestruct *real_current,
 	}
 #endif
 
-	if (!replaceall)
+	if (!replaceall) {
+	    placewewant = xplustabs();
 	    edit_redraw(current_save, old_pww);
+	    old_pww = placewewant;
+	}
 
 #ifdef HAVE_REGEX_H
 	if (ISSET(USE_REGEXP))
@@ -681,7 +685,7 @@ int do_replace_loop(const char *needle, const filestruct *real_current,
 	    int length_change;
 
 	    if (i == 2)
-		replaceall = 1;
+		replaceall = TRUE;
 
 	    copy = replace_line(needle);
 
@@ -743,7 +747,7 @@ int do_replace_loop(const char *needle, const filestruct *real_current,
 }
 
 /* Replace a string. */
-int do_replace(void)
+void do_replace(void)
 {
     int i, numreplaced;
     filestruct *edittop_save, *begin;
@@ -752,23 +756,23 @@ int do_replace(void)
     if (ISSET(VIEW_MODE)) {
 	print_view_warning();
 	replace_abort();
-	return 0;
+	return;
     }
 
     i = search_init(1);
     if (i == -1) {		/* Cancel, Go to Line, blank search
 				 * string, or regcomp() failed. */
 	replace_abort();
-	return 0;
+	return;
     } else if (i == -2) {	/* No Replace. */
 	do_search();
-	return 0;
+	return;
     } else if (i == 1)		/* Case Sensitive, Backwards, or Regexp
 				 * search toggle. */
 	do_replace();
 
     if (i != 0)
-	return 0;
+	return;
 
     /* If answer is not "", add answer to the search history list and
      * copy answer into last_search. */
@@ -804,7 +808,7 @@ int do_replace(void)
 	    statusbar(_("Replace Cancelled"));
 	}
 	replace_abort();
-	return 0;
+	return;
     }
 
     last_replace = mallocstrcpy(last_replace, answer);
@@ -818,24 +822,24 @@ int do_replace(void)
     numreplaced = do_replace_loop(last_search, begin, &beginx, FALSE);
 
     /* Restore where we were. */
+    edittop = edittop_save;
     current = begin;
     current_x = beginx;
     renumber_all();
-    edit_update(edittop_save, TOP);
+    edit_refresh();
 
     if (numreplaced >= 0)
 	statusbar(P_("Replaced %d occurrence", "Replaced %d occurrences",
 		numreplaced), numreplaced);
 
     replace_abort();
-    return 1;
 }
 
-int do_gotoline(int line, int save_pos)
+void do_gotoline(int line, bool save_pos)
 {
-    if (line <= 0) {		/* Ask for it */
+    if (line <= 0) {		/* Ask for it. */
 	char *ans = mallocstrcpy(NULL, answer);
-	int st = statusq(FALSE, goto_list, line != 0 ? ans : "",
+	int st = statusq(FALSE, goto_list, line < 0 ? ans : "",
 #ifndef NANO_SMALL
 		NULL,
 #endif
@@ -848,64 +852,70 @@ int do_gotoline(int line, int save_pos)
 	    statusbar(_("Aborted"));
 	if (st != 0) {
 	    display_main_list();
-	    return 0;
+	    return;
 	}
 
-	line = atoi(answer);
-
 	/* Bounds check. */
-	if (line <= 0) {
+	if (!parse_num(answer, &line) || line < 0) {
 	    statusbar(_("Come on, be reasonable"));
 	    display_main_list();
-	    return 0;
+	    return;
 	}
     }
 
-    for (current = fileage; current->next != NULL && line > 1; line--)
-	current = current->next;
+    if (current->lineno > line) {
+	for (; current->prev != NULL && current->lineno > line;
+		current = current->prev)
+	    ;
+    } else {
+	for (; current->next != NULL && current->lineno < line;
+		current = current->next)
+	    ;
+    }
 
     current_x = 0;
 
-    /* If save_pos is nonzero, don't change the cursor position when
+    /* If save_pos is TRUE, don't change the cursor position when
      * updating the edit window. */
     edit_update(current, save_pos ? NONE : CENTER);
 
     placewewant = 0;
     display_main_list();
-    return 1;
 }
 
-int do_gotoline_void(void)
+void do_gotoline_void(void)
 {
-    return do_gotoline(0, 0);
+    do_gotoline(0, FALSE);
 }
 
 #if defined(ENABLE_MULTIBUFFER) || !defined(DISABLE_SPELLER)
-void do_gotopos(int line, int pos_x, int pos_y, int pos_placewewant)
+void do_gotopos(int line, int pos_x, int pos_y, size_t pos_pww)
 {
-    /* since do_gotoline() resets the x-coordinate but not the
-       y-coordinate, set the coordinates up this way */
+    /* Since do_gotoline() resets the x-coordinate but not the
+     * y-coordinate, set the coordinates up this way. */
     current_y = pos_y;
-    do_gotoline(line, 1);
+    do_gotoline(line, TRUE);
 
-    /* make sure that the x-coordinate is sane here */
-    if (pos_x > strlen(current->data))
-	pos_x = strlen(current->data);
+    /* Make sure that the x-coordinate is sane here. */
+    current_x = strlen(current->data);
+    if (pos_x < current_x)
+	current_x = pos_x;
 
-    /* set the rest of the coordinates up */
-    current_x = pos_x;
-    placewewant = pos_placewewant;
+    /* Set the rest of the coordinates up. */
+    placewewant = pos_pww;
     update_line(current, pos_x);
 }
 #endif
 
 #if !defined(NANO_SMALL) && defined(HAVE_REGEX_H)
-int do_find_bracket(void)
+void do_find_bracket(void)
 {
     char ch_under_cursor, wanted_ch;
     const char *pos, *brackets = "([{<>}])";
     char regexp_pat[] = "[  ]";
-    int old_pww = placewewant, current_x_save, flagsave, count = 1;
+    size_t old_pww, current_x_save;
+    int count = 1;
+    long flags_save;
     filestruct *current_save;
 
     ch_under_cursor = current->data[current_x];
@@ -913,25 +923,28 @@ int do_find_bracket(void)
     pos = strchr(brackets, ch_under_cursor);
     if (ch_under_cursor == '\0' || pos == NULL) {
 	statusbar(_("Not a bracket"));
-	return 1;
+	return;
     }
 
     assert(strlen(brackets) % 2 == 0);
     wanted_ch = brackets[(strlen(brackets) - 1) - (pos - brackets)];
 
+    old_pww = placewewant;
     current_x_save = current_x;
     current_save = current;
-    flagsave = flags;
+    flags_save = flags;
     SET(USE_REGEXP);
 
     /* Apparent near redundancy with regexp_pat[] here is needed.
      * "[][]" works, "[[]]" doesn't. */
 
-    if (pos < brackets + (strlen(brackets) / 2)) {	/* On a left bracket. */
+    if (pos < brackets + (strlen(brackets) / 2)) {
+	/* On a left bracket. */
 	regexp_pat[1] = wanted_ch;
 	regexp_pat[2] = ch_under_cursor;
 	UNSET(REVERSE_SEARCH);
-    } else {			/* On a right bracket. */
+    } else {
+	/* On a right bracket. */
 	regexp_pat[1] = ch_under_cursor;
 	regexp_pat[2] = wanted_ch;
 	SET(REVERSE_SEARCH);
@@ -943,7 +956,8 @@ int do_find_bracket(void)
 
     search_last_line = FALSE;
     while (TRUE) {
-	if (findnextstr(FALSE, FALSE, current, current_x, regexp_pat, FALSE) != 0) {
+	if (findnextstr(FALSE, FALSE, current, current_x, regexp_pat,
+		FALSE) != 0) {
 	    /* Found identical bracket. */
 	    if (current->data[current_x] == ch_under_cursor)
 		count++;
@@ -964,8 +978,7 @@ int do_find_bracket(void)
     }
 
     regexp_cleanup();
-    flags = flagsave;
-    return 0;
+    flags = flags_save;
 }
 #endif
 
@@ -993,7 +1006,7 @@ void history_init(void)
 }
 
 /* find first node containing string *s in history list *h */
-historytype *find_node(historytype *h, char *s)
+historytype *find_node(historytype *h, const char *s)
 {
     for (; h->next != NULL; h = h->next)
 	if (strcmp(s, h->data) == 0)
@@ -1017,14 +1030,14 @@ void insert_node(historytype *h, const char *s)
 
     a = (historytype *)nmalloc(sizeof(historytype));
     a->next = h->next;
-    a->prev = h->next->prev;
+    a->prev = h;
     h->next->prev = a;
     h->next = a;
     a->data = mallocstrcpy(NULL, s);
 }
 
 /* update history list */
-void update_history(historyheadtype *h, char *s)
+void update_history(historyheadtype *h, const char *s)
 {
     historytype *p;
 
@@ -1082,14 +1095,16 @@ char *get_history_completion(historyheadtype *h, char *s)
     return s;
 }
 
+#ifdef DEBUG
 /* free a history list */
 void free_history(historyheadtype *h)
 {
-    historytype *p, *n;
+    historytype *p;
 
-    for (p = h->next; (n = p->next); p = n)
+    for (p = h->next; p->next != NULL; p = p->next)
 	remove_node(p);
 }
+#endif
 
 /* end of history support functions */
 #endif /* !NANO_SMALL */
