@@ -1,4 +1,4 @@
-/* $Id: utils.c,v 1.13 2004/05/23 21:11:14 dolorous Exp $ */
+/* $Id: utils.c,v 1.23 2004/08/17 20:34:57 dolorous Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
@@ -21,11 +21,12 @@
 
 #include "config.h"
 
-#include <stdio.h>
-#include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
+#include <unistd.h>
 #include <ctype.h>
+#include <errno.h>
 #include <assert.h>
 #include "proto.h"
 #include "nano.h"
@@ -83,6 +84,23 @@ int num_of_digits(int n)
     return i;
 }
 
+/* Read an int from str, and store it in *val (if val is not NULL).  On
+ * error, we return FALSE and don't change *val.  Otherwise, we return 
+ * TRUE. */
+bool parse_num(const char *str, ssize_t *val)
+{
+    char *first_error;
+    ssize_t j;
+
+    assert(str != NULL);
+    j = (ssize_t)strtol(str, &first_error, 10);
+    if (errno == ERANGE || *str == '\0' || *first_error != '\0')
+	return FALSE;
+    if (val != NULL)
+	*val = j;
+    return TRUE;
+}
+
 /* Fix the memory allocation for a string. */
 void align(char **strp)
 {
@@ -149,10 +167,8 @@ int nstrnicmp(const char *s1, const char *s2, size_t n)
 
     if (n > 0)
 	return (tolower(*s1) - tolower(*s2));
-    else if (n == 0)
-	return 0;
     else
-	return -1;
+	return 0;
 }
 #endif
 
@@ -224,6 +240,66 @@ size_t nstrnlen(const char *s, size_t maxlen)
 }
 #endif
 
+#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+#ifndef HAVE_GETLINE
+/* This function is equivalent to getline().  It was adapted from
+ * GNU mailutils' getline() function. */
+ssize_t ngetline(char **lineptr, size_t *n, FILE *stream)
+{
+    return getdelim(lineptr, n, '\n', stream);
+}
+#endif
+
+#ifndef HAVE_GETDELIM
+/* This function is equivalent to getdelim().  It was adapted from
+ * GNU mailutils' getdelim() function. */
+ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
+{
+    size_t indx = 0;
+    int c;
+
+    /* Sanity checks. */
+    if (lineptr == NULL || n == NULL || stream == NULL)
+	return -1;
+
+    /* Allocate the line the first time. */
+    if (*lineptr == NULL) {
+	*lineptr = charalloc(128);
+	*n = 128;
+    }
+
+    while ((c = getc(stream)) != EOF) {
+	/* Check if more memory is needed. */
+	if (indx >= *n) {
+	    *lineptr = charealloc(*lineptr, *n + 128);
+	    *n += 128;
+	}
+
+	/* Push the result in the line. */
+	(*lineptr)[indx++] = (char)c;
+
+	/* Bail out. */
+	if (c == delim)
+	    break;
+    }
+
+    /* Make room for the null character. */
+    if (indx >= *n) {
+	*lineptr = charealloc(*lineptr, *n + 128);
+	*n += 128;
+    }
+
+    /* Null terminate the buffer. */
+    null_at(lineptr, indx++);
+    *n = indx;
+
+    /* The last line may not have the delimiter, we have to return what
+     * we got and the error will be seen on the next iteration. */
+    return (c == EOF && (indx - 1) == 0) ? -1 : indx - 1;
+}
+#endif
+#endif /* !NANO_SMALL && ENABLE_NANORC */
+
 /* If we are searching backwards, we will find the last match that
  * starts no later than start.  Otherwise we find the first match
  * starting no earlier than start.  If we are doing a regexp search, we
@@ -352,8 +428,8 @@ void new_magicline(void)
 #ifndef NANO_SMALL
 /* Set top_x and bot_x to the top and bottom x-coordinates of the mark,
  * respectively, based on the locations of top and bot. */
-void mark_order(const filestruct **top, size_t *top_x,
-		const filestruct **bot, size_t *bot_x)
+void mark_order(const filestruct **top, size_t *top_x, const filestruct
+	**bot, size_t *bot_x)
 {
     assert(top != NULL && top_x != NULL && bot != NULL && bot_x != NULL);
     if ((current->lineno == mark_beginbuf->lineno && current_x > mark_beginx)
@@ -420,10 +496,10 @@ int check_wildcard_match(const char *text, const char *pattern)
 		    found = TRUE;
 	    }
 	    len = strlen(text);
-	    if (found == FALSE && len != 0) {
+	    if (!found && len != 0) {
 		return FALSE;
 	    }
-	    if (found == TRUE) {
+	    if (found) {
 		if (strlen(pattern) == 0 && len == 1) {
 		    return TRUE;
 		}
