@@ -1,4 +1,4 @@
-/* $Id: winio.c,v 1.100 2004/06/26 15:24:16 dolorous Exp $ */
+/* $Id: winio.c,v 1.113 2004/08/01 22:35:31 dolorous Exp $ */
 /**************************************************************************
  *   winio.c                                                              *
  *                                                                        *
@@ -193,7 +193,8 @@ int get_translated_kbinput(int kbinput, int *es
 #endif
 	)
 {
-    static size_t escapes = 0, ascii_digits = 0;
+    static int escapes = 0;
+    static size_t ascii_digits = 0;
     int retval = ERR;
 
 #ifndef NANO_SMALL
@@ -413,9 +414,9 @@ int get_translated_kbinput(int kbinput, int *es
 		    }
 	    }
     }
-
+ 
 #ifdef DEBUG
-    fprintf(stderr, "get_translated_kbinput(): kbinput = %d, es = %d, escapes = %d, ascii_digits = %d, retval = %d\n", kbinput, *es, escapes, ascii_digits, retval);
+    fprintf(stderr, "get_translated_kbinput(): kbinput = %d, es = %d, escapes = %d, ascii_digits = %lu, retval = %d\n", kbinput, *es, escapes, (unsigned long)ascii_digits, retval);
 #endif
 
     /* Return the result. */
@@ -511,7 +512,7 @@ int get_ascii_kbinput(int kbinput, size_t ascii_digits
     }
 
 #ifdef DEBUG
-    fprintf(stderr, "get_ascii_kbinput(): kbinput = %d, ascii_digits = %d, ascii_kbinput = %d, retval = %d\n", kbinput, ascii_digits, ascii_kbinput, retval);
+    fprintf(stderr, "get_ascii_kbinput(): kbinput = %d, ascii_digits = %lu, ascii_kbinput = %d, retval = %d\n", kbinput, (unsigned long)ascii_digits, ascii_kbinput, retval);
 #endif
 
     /* If the result is an ASCII character, reset the ASCII character
@@ -550,7 +551,7 @@ int get_control_kbinput(int kbinput)
 	retval = kbinput;
 
 #ifdef DEBUG
-    fprintf(stderr, "get_control_kbinput(): kbinput = %d, retval = %d\n", kbinput, ascii_digits, *complete, retval);
+    fprintf(stderr, "get_control_kbinput(): kbinput = %d, retval = %d\n", kbinput, retval);
 #endif
 
     return retval;
@@ -1069,8 +1070,8 @@ int get_escape_seq_abcd(int kbinput)
 /* Read in a string of input characters (e.g. an escape sequence)
  * verbatim.  Store the string in v_kbinput and return the length
  * of the string in v_len.  Assume nodelay(win) is FALSE. */
-int *get_verbatim_kbinput(WINDOW *win, int *v_kbinput, size_t
-	*v_len, int allow_ascii)
+int *get_verbatim_kbinput(WINDOW *win, int *v_kbinput, size_t *v_len,
+	int allow_ascii)
 {
     int kbinput;
     size_t i = 0, v_newlen = 0;
@@ -1096,7 +1097,7 @@ int *get_verbatim_kbinput(WINDOW *win, int *v_kbinput, size_t
     (*v_len)++;
     v_kbinput[0] = kbinput;
 #ifdef DEBUG
-    fprintf(stderr, "get_verbatim_kbinput(): kbinput = %d, v_len = %d\n", kbinput, *v_len);
+    fprintf(stderr, "get_verbatim_kbinput(): kbinput = %d, v_len = %lu\n", kbinput, (unsigned long)*v_len);
 #endif
 
     /* Read any following characters using non-blocking input, until
@@ -1111,7 +1112,7 @@ int *get_verbatim_kbinput(WINDOW *win, int *v_kbinput, size_t
 	v_kbinput = (int *)nrealloc(v_kbinput, *v_len * sizeof(int));
 	v_kbinput[*v_len - 1] = kbinput;
 #ifdef DEBUG
-	fprintf(stderr, "get_verbatim_kbinput(): kbinput = %d, v_len = %d\n", kbinput, *v_len);
+	fprintf(stderr, "get_verbatim_kbinput(): kbinput = %d, v_len = %lu\n", kbinput, (unsigned long)*v_len);
 #endif
     }
     nodelay(win, FALSE);
@@ -1230,7 +1231,7 @@ int get_untranslated_kbinput(int kbinput, size_t position, int
      retval = kbinput;
 
 #ifdef DEBUG
-    fprintf(stderr, "get_untranslated_kbinput(): kbinput = %d, position = %d, ascii_digits = %d\n", kbinput, position, ascii_digits);
+    fprintf(stderr, "get_untranslated_kbinput(): kbinput = %d, position = %lu, ascii_digits = %lu\n", kbinput, (unsigned long)position, (unsigned long)ascii_digits);
 #endif
 
     return retval;
@@ -1239,12 +1240,12 @@ int get_untranslated_kbinput(int kbinput, size_t position, int
 #ifndef DISABLE_MOUSE
 /* Check for a mouse event, and if one's taken place, save the
  * coordinates where it took place in mouse_x and mouse_y.  After that,
- * if allow_shortcuts is zero, return 0.  Otherwise, if the mouse event
- * took place on the shortcut list on the bottom two lines of the screen
- * (assuming that the shortcut list is visible), figure out which
- * shortcut was clicked and ungetch() the equivalent keystroke(s).
- * Return 0 if no keystrokes were ungetch()ed, or 1 if at least one was.
- * Assume that KEY_MOUSE has already been read in. */
+ * assuming allow_shortcuts is FALSE, if the shortcut list on the
+ * bottom two lines of the screen is visible and the mouse event took
+ * place on it, figure out which shortcut was clicked and ungetch() the 
+ * equivalent keystroke(s).  Return FALSE if no keystrokes were 
+ * ungetch()ed, or TRUE if at least one was.  Assume that KEY_MOUSE has 
+ * already been read in. */
 int get_mouseinput(int *mouse_x, int *mouse_y, int allow_shortcuts)
 {
     MEVENT mevent;
@@ -1254,7 +1255,7 @@ int get_mouseinput(int *mouse_x, int *mouse_y, int allow_shortcuts)
 
     /* First, get the actual mouse event. */
     if (getmouse(&mevent) == ERR)
-	return 0;
+	return FALSE;
 
     /* Save the screen coordinates where the mouse event took place. */
     *mouse_x = mevent.x;
@@ -1262,7 +1263,7 @@ int get_mouseinput(int *mouse_x, int *mouse_y, int allow_shortcuts)
 
     /* If we're not allowing shortcuts' we're done now. */
     if (!allow_shortcuts)
-	return 0;
+	return FALSE;
 
     /* Otherwise, if the current shortcut list is being displayed on the
      * last two lines of the screen and the mouse event took place
@@ -1270,7 +1271,7 @@ int get_mouseinput(int *mouse_x, int *mouse_y, int allow_shortcuts)
      * ungetch() the equivalent keystroke(s) for it. */
     if (!ISSET(NO_HELP) && wenclose(bottomwin, *mouse_y, *mouse_x)) {
 	int i, j;
-	int currslen;
+	size_t currslen;
 	    /* The number of shortcuts in the current shortcut list. */
 	const shortcut *s = currshortcut;
 	    /* The actual shortcut we clicked on, starting at the first
@@ -1318,9 +1319,9 @@ int get_mouseinput(int *mouse_x, int *mouse_y, int allow_shortcuts)
 	    ungetch(NANO_CONTROL_3);
 	}
 
-	return 1;
+	return TRUE;
     }
-    return 0;
+    return FALSE;
 }
 #endif
 
@@ -1392,7 +1393,7 @@ void blank_titlebar(void)
 
 void blank_edit(void)
 {
-    size_t i;
+    int i;
     for (i = 0; i < editwinrows; i++)
 	mvwaddstr(edit, i, 0, hblank);
 }
@@ -1535,7 +1536,7 @@ int nanogetstr(int allowtabs, const char *buf, const char *def,
 #endif
 		const shortcut *s
 #ifndef DISABLE_TABCOMP
-		, int *list
+		, bool *list
 #endif
 		)
 {
@@ -1545,7 +1546,7 @@ int nanogetstr(int allowtabs, const char *buf, const char *def,
 	/* the cursor position in 'answer' */
     int xend;
 	/* length of 'answer', the status bar text */
-    int tabbed = 0;
+    bool tabbed = FALSE;
 	/* used by input_tab() */
     const shortcut *t;
 
@@ -1589,7 +1590,8 @@ int nanogetstr(int allowtabs, const char *buf, const char *def,
 
     /* Make sure any editor screen updates are displayed before getting
        input */
-    wrefresh(edit);
+    wnoutrefresh(edit);
+    wrefresh(bottomwin);
 
     /* If we're using restricted mode, we aren't allowed to change the
      * name of a file once it has one because that would allow writing
@@ -1628,7 +1630,7 @@ int nanogetstr(int allowtabs, const char *buf, const char *def,
 	assert(0 <= x && x <= xend && xend == strlen(answer));
 
 	if (kbinput != '\t')
-	    tabbed = 0;
+	    tabbed = FALSE;
 
 	switch (kbinput) {
 #ifndef DISABLE_MOUSE
@@ -1668,10 +1670,8 @@ int nanogetstr(int allowtabs, const char *buf, const char *def,
 	    }
 	    break;
 	case NANO_CUT_KEY:
-	case NANO_UNCUT_KEY:
 	    /* If we're using restricted mode, the filename isn't blank,
-	     * and we're at the "Write File" prompt, disable Cut and
-	     * UnCut. */
+	     * and we're at the "Write File" prompt, disable Cut. */
 	    if (!ISSET(RESTRICTED) || filename[0] == '\0' || s != writefile_list) {
 		null_at(&answer, 0);
 		xend = 0;
@@ -1736,7 +1736,8 @@ int nanogetstr(int allowtabs, const char *buf, const char *def,
 		   search history, and we need to save the current
 		   answer in currentbuf; do this and reset use_cb to
 		   0 */
-		if (currentbuf == NULL || (use_cb == 1 && strcmp(currentbuf, answer))) {
+		if (currentbuf == NULL || (use_cb == 1 &&
+			strcmp(currentbuf, answer) != 0)) {
 		    currentbuf = mallocstrcpy(currentbuf, answer);
 		    use_cb = 0;
 		}
@@ -1747,7 +1748,8 @@ int nanogetstr(int allowtabs, const char *buf, const char *def,
 		   we need to make the string in currentbuf the current
 		   answer; do this, blow away currentbuf since we don't
 		   need it anymore, and reset use_cb to 0 */
-		if (currentbuf != NULL && use_cb == 2 && strcmp(currentbuf, answer)) {
+		if (currentbuf != NULL && use_cb == 2 &&
+			strcmp(currentbuf, answer) != 0) {
 		    answer = mallocstrcpy(answer, currentbuf);
 		    free(currentbuf);
 		    currentbuf = NULL;
@@ -1818,7 +1820,7 @@ int nanogetstr(int allowtabs, const char *buf, const char *def,
 		    fprintf(stderr, "Aha! \'%c\' (%d)\n", kbinput,
 			    kbinput);
 #endif
-		    if (meta_key == TRUE && (kbinput == t->metaval || kbinput == t->miscval))
+		    if (meta_key && (kbinput == t->metaval || kbinput == t->miscval))
 			/* We hit a meta key.  Do like above.  We don't
 			 * just ungetch() the letter and let it get
 			 * caught above cause that screws the
@@ -2182,7 +2184,6 @@ void reset_cursor(void)
     current_y = current->lineno - edittop->lineno;
     if (current_y < editwinrows) {
 	size_t x = xplustabs();
-
 	wmove(edit, current_y, x - get_page_start(x));
      }
 }
@@ -2545,7 +2546,7 @@ void update_line(const filestruct *fileptr, size_t index)
 /* Return a nonzero value if we need an update after moving
  * horizontally.  We need one if the mark is on or if old_pww and
  * placewewant are on different pages. */
-int need_horizontal_update(int old_pww)
+int need_horizontal_update(size_t old_pww)
 {
     return
 #ifndef NANO_SMALL
@@ -2557,7 +2558,7 @@ int need_horizontal_update(int old_pww)
 /* Return a nonzero value if we need an update after moving vertically.
  * We need one if the mark is on or if old_pww and placewewant
  * are on different pages. */
-int need_vertical_update(int old_pww)
+int need_vertical_update(size_t old_pww)
 {
     return
 #ifndef NANO_SMALL
@@ -2632,7 +2633,7 @@ void edit_scroll(updown direction, int nlines)
 
 /* Update any lines between old_current and current that need to be
  * updated. */
-void edit_redraw(const filestruct *old_current, int old_pww)
+void edit_redraw(const filestruct *old_current, size_t old_pww)
 {
     int do_refresh = need_vertical_update(0) ||
 	need_vertical_update(old_pww);
@@ -2691,7 +2692,7 @@ void edit_refresh(void)
 	const filestruct *foo = edittop;
 
 #ifdef DEBUG
-	fprintf(stderr, "edit_refresh(): edittop->lineno = %ld\n", edittop->lineno);
+	fprintf(stderr, "edit_refresh(): edittop->lineno = %d\n", edittop->lineno);
 #endif
 
 	while (nlines < editwinrows) {
@@ -2706,9 +2707,6 @@ void edit_refresh(void)
 	    nlines++;
 	}
 	reset_cursor();
-
-	/* What the hell are we expecting to update the screen if this
-	 * isn't here?  Luck? */
 	wrefresh(edit);
     }
 }
@@ -2746,7 +2744,7 @@ int statusq(int allowtabs, const shortcut *s, const char *def,
     char *foo = charalloc(COLS - 3);
     int ret;
 #ifndef DISABLE_TABCOMP
-    int list = 0;
+    bool list = FALSE;
 #endif
 
     bottombars(s);
@@ -2887,7 +2885,7 @@ int do_yesno(int all, const char *msg)
 	/* Look ma!  We get to duplicate lots of code from
 	 * do_mouse()!! */
 	else if (kbinput == KEY_MOUSE) {
-	    kbinput = get_mouseinput(&mouse_x, &mouse_y, 0);
+	    kbinput = get_mouseinput(&mouse_x, &mouse_y, FALSE);
 
 	    if (mouse_x != -1 && mouse_y != -1 && !ISSET(NO_HELP) &&
 		wenclose(bottomwin, mouse_y, mouse_x) && mouse_x <
@@ -2923,7 +2921,7 @@ int do_yesno(int all, const char *msg)
     return ok;
 }
 
-int total_refresh(void)
+void total_refresh(void)
 {
     clearok(topwin, TRUE);
     clearok(edit, TRUE);
@@ -2937,7 +2935,6 @@ int total_refresh(void)
     clearok(bottomwin, FALSE);
     edit_refresh();
     titlebar(NULL);
-    return 1;
 }
 
 void display_main_list(void)
@@ -2945,14 +2942,14 @@ void display_main_list(void)
     bottombars(main_list);
 }
 
-/* If constant is FALSE, the user typed ^C so we unconditionally display
- * the cursor position.  Otherwise, we display it only if the character
- * position changed, and DISABLE_CURPOS is not set.
+/* If constant is FALSE, the user typed Ctrl-C, so we unconditionally
+ * display the cursor position.  Otherwise, we display it only if the
+ * character position changed and DISABLE_CURPOS is not set.
  *
  * If constant is TRUE and DISABLE_CURPOS is set, we unset it and update
  * old_i and old_totsize.  That way, we leave the current statusbar
  * alone, but next time we will display. */
-int do_cursorpos(int constant)
+void do_cursorpos(int constant)
 {
     const filestruct *fileptr;
     unsigned long i = 0;
@@ -2970,19 +2967,23 @@ int do_cursorpos(int constant)
     }
     i += current_x;
 
+    /* Check whether totsize is correct.  Else there is a bug
+     * somewhere. */
+    assert(current != filebot || i == totsize);
+
     if (constant && ISSET(DISABLE_CURPOS)) {
 	UNSET(DISABLE_CURPOS);
 	old_i = i;
 	old_totsize = totsize;
-	return 0;
+	return;
     }
 
     /* If constant is FALSE, display the position on the statusbar
      * unconditionally; otherwise, only display the position when the
      * character values have changed. */
     if (!constant || old_i != i || old_totsize != totsize) {
-	unsigned long xpt = xplustabs() + 1;
-	unsigned long cur_len = strlenpt(current->data) + 1;
+	size_t xpt = xplustabs() + 1;
+	size_t cur_len = strlenpt(current->data) + 1;
 	int linepct = 100 * current->lineno / totlines;
 	int colpct = 100 * xpt / cur_len;
 	int bytepct = totsize == 0 ? 0 : 100 * i / totsize;
@@ -2990,25 +2991,23 @@ int do_cursorpos(int constant)
 	statusbar(
 	    _("line %ld/%ld (%d%%), col %lu/%lu (%d%%), char %lu/%ld (%d%%)"),
 		    current->lineno, totlines, linepct,
-		    xpt, cur_len, colpct,
+		    (unsigned long)xpt, (unsigned long)cur_len, colpct,
 		    i, totsize, bytepct);
 	UNSET(DISABLE_CURPOS);
     }
 
     old_i = i;
     old_totsize = totsize;
-
-    reset_cursor();
-    return 0;
 }
 
-int do_cursorpos_void(void)
+void do_cursorpos_void(void)
 {
-    return do_cursorpos(0);
+    do_cursorpos(FALSE);
 }
 
+#ifndef DISABLE_HELP
 /* Calculate the next line of help_text, starting at ptr. */
-int line_len(const char *ptr)
+int help_line_len(const char *ptr)
 {
     int j = 0;
 
@@ -3033,9 +3032,8 @@ int line_len(const char *ptr)
     return j;
 }
 
-#ifndef DISABLE_HELP
 /* Our dynamic, shortcut-list-compliant help function. */
-int do_help(void)
+void do_help(void)
 {
     int line = 0;
 	/* The line number in help_text of the first displayed help line.
@@ -3120,13 +3118,13 @@ int do_help(void)
 	/* Calculate where in the text we should be, based on the
 	 * page. */
 	for (i = 0; i < line; i++) {
-	    ptr += line_len(ptr);
+	    ptr += help_line_len(ptr);
 	    if (*ptr == '\n')
 		ptr++;
 	}
 
 	for (i = 0; i < editwinrows && *ptr != '\0'; i++) {
-	    int j = line_len(ptr);
+	    int j = help_line_len(ptr);
 
 	    mvwaddnstr(edit, i, 0, ptr, j);
 	    ptr += j;
@@ -3137,7 +3135,8 @@ int do_help(void)
 
   skip_redisplay:
 	kbinput = get_kbinput(edit, &meta_key);
-    } while (kbinput != NANO_EXIT_KEY && kbinput != NANO_EXIT_FKEY);
+    } while (kbinput != NANO_CANCEL_KEY && kbinput != NANO_EXIT_KEY &&
+	kbinput != NANO_EXIT_FKEY);
 
 #ifndef DISABLE_MOUSE
     currshortcut = oldshortcut;
@@ -3159,8 +3158,6 @@ int do_help(void)
      * anymore. */
     free(help_text);
     help_text = NULL;
-
-    return 1;
 }
 #endif /* !DISABLE_HELP */
 
@@ -3168,7 +3165,7 @@ int do_help(void)
  * expect word to have tabs and control characters expanded. */
 void do_replace_highlight(int highlight_flag, const char *word)
 {
-    int y = xplustabs();
+    size_t y = xplustabs();
     size_t word_len = strlen(word);
 
     y = get_page_start(y) + COLS - y;
@@ -3291,14 +3288,14 @@ void do_credits(void)
     };
 
     const char *xlcredits[XLCREDIT_LEN] = {
-	"The nano text editor",
-	"version",
-	"Brought to you by:",
-	"Special thanks to:",
-	"The Free Software Foundation",
-	"For ncurses:",
-	"and anyone else we forgot...",
-	"Thank you for using nano!"
+	N_("The nano text editor"),
+	N_("version"),
+	N_("Brought to you by:"),
+	N_("Special thanks to:"),
+	N_("The Free Software Foundation"),
+	N_("For ncurses:"),
+	N_("and anyone else we forgot..."),
+	N_("Thank you for using nano!")
     };
 
     curs_set(0);
